/**
 ** infusion - The Desktop Communications Revoution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: cxcachemgr.o
 ** Date: 2001-08-23
 ** Last Revision: 2001-08-24
 ** Description: CxConnectoin cache manager.
 ** CVS: $Id: cxcachemgr.cpp,v 1.1 2001/08/28 01:20:58 brian Exp $
 **/
#include	<iostream>
#include	<stdio.h>
#include	<qobject.h>
#include	<qtimer.h>
#include	<qdict.h>
#include	<qlist.h>
#include	<time.h>
#include	<dcopclient.h>
#include	<kdebug.h>
#include	<qstring.h>
#include	"cxcachemgr.h"
#include	"cxconnection.h"

/**
 ** CxCacheMgr::CxCacheMgr(): Start up and allocate space for my initial
 ** data set.
 **/
		CxCacheMgr::CxCacheMgr( CxConnection *parent ) {

	kdDebug() << "CxCacheMgr::CxCacheMgr(): Created new CacheMgr instance @" << this << endl;
	kdDebug() << "CxCacheMgr::CxCacheMgr(): * Attached to parent @" << parent << endl;
	g_Connection = parent;

	maintTimer = new QTimer( this );
	connect( maintTimer, SIGNAL(timeout()), this, SLOT(_cacheMaintenance()));
	maintTimer->changeInterval(50000);
}

/**
 ** CxCacheMgr::~CxCacheMgr(): DESTRUCTOR.  Flush all tables and destroy
 ** all tables.
 **/
		CxCacheMgr::~CxCacheMgr() {

	kdDebug() << "CxCacheMgr::~CxCacheMgr(): Destroying CacheMgr instance @" << this << endl;
}

/**
 ** CxCacheMgr::exists(): The room/message you requested exists.
 **/
bool		CxCacheMgr::exists( const QString& room, long unsigned int msg_id ) {

	kdDebug() << "CxCacheMgr::exists(): Locate \"" << room << ":" << msg_id << "\"" << endl;
	if( msg_id ) {
		return( _messageExists( room, msg_id ));
	} else {
		return( _roomExists( room ));
	}
}

/**
 ** CxCacheMgr::isComplete(): Does the specified message have a complete set of
 ** header & body data?
 **/
bool		CxCacheMgr::isComplete( const QString&, long unsigned int ) {

	kdDebug() << "CxCacheMgr::isComplete(): STUB" << endl;
	return( false );
}

/**
 ** CxCacheMgr::isFresh(): Is the referenced room/message "fresh" in cache?
 **/
bool		CxCacheMgr::isFresh( const QString& room, long unsigned int msg_id ) {

	if( msg_id ) {
		return( _messageFresh( room, msg_id ) );
	} else {
		return( _roomFresh( room ) );
	}
}

/**
 ** CxCacheMgr::insert(): Insert a new message into the cache.  This may just be
 ** a set of headers, but we can store it in cache anyway (with a different flag)
 **/
bool		CxCacheMgr::insert( const QString& room, long unsigned int msg_id, QMesgInfo *minf ) {

	kdDebug() << "CxCacheMgr::insert(): Requested an insert operation" << endl;
	kdDebug() << "CxCacheMgr::insert(): GUID: \"" << room << ":" << msg_id << "\"" << endl;

	if( msg_id && minf ) {
		return( _insertMessage( room, msg_id, minf ) );
	} else {
		return( _insertRoom( room ) );
	}
}

/**
 ** CxCacheMgr::fetch(): Retrieve a message from the cache.  If the message
 ** is incomplete, optionally fetch it from the host.
 **/
QMesgInfo	*CxCacheMgr::fetch( const QString& room, long unsigned int msg_id, unsigned int flags ) {
char                    mid[255];

	kdDebug() << "CxCacheMgr::fetch(): GUID: \"" << room << ":" << msg_id << "\"" << endl;
	kdDebug() << "CxCacheMgr::fetch(): (flags " << flags << ")" << endl;

	if( room.isNull() || !msg_id ) {
		kdDebug() << "CxCacheMgr::fetch(): Unable to load unspecified message." << endl;
		return( 0L );
	}

	sprintf( mid, "%ld", msg_id );

	if( g_RoomCache.find( room ) && (g_RoomCache.find( room ))->find( mid ) ) {

		if( ((g_RoomCache.find( room ))->find( mid ))->MessageId != msg_id ) {
			kdDebug() << "CxCacheMgr::fetch(): " << ((g_RoomCache.find( room ))->find( mid ))->MessageId <<
				" != " << msg_id << endl;
			kdDebug() << "CxCacheMgr::fetch(): MessageId mismatch??  Deleting corrupt entry." << endl;
			(g_RoomCache.find( room ))->remove( mid );
			return(0L);
		}

		kdDebug() << "CxCacheMgr::fetch(): Found room @" << (g_RoomCache.find( room )) << endl;
		kdDebug() << "CxCacheMgr::fetch(): Found msg @" << ((g_RoomCache.find( room ))->find( mid )) << endl;
		kdDebug() << "CxCacheMgr::fetch(): m->flags: " << ((g_RoomCache.find( room ))->find( mid ))->flags 
			<< endl;
		kdDebug() << "CxCacheMgr::fetch(): Message intact, returning." << endl;

//		if( (((g_RoomCache.find( room ))->find( mid ))->flags & (MSG_HEADER|MSG_BODY)) != (MSG_HEADER|MSG_BODY)) {
/*		if( !((g_RoomCache.find( room ))->find( mid ))->flags ) {
			QMesgInfo *foo = g_Connection->fetchMesgInfo( QCString(room), msg_id, false );
			_insertMessage( room, msg_id, foo );
		} */

		return((g_RoomCache.find( room ))->find( mid ));

	} else {
		kdDebug() << "CxCacheMgr::fetch(): NOT Found.  Stack:" << endl;
		kdDebug() << "CxCacheMgr::fetch(): 1: " << g_RoomCache.find( room ) << endl;
		if(g_RoomCache.find(room))
			kdDebug() << "CxCacheMgr::fetch(): 2: " <<
				(g_RoomCache.find( room ))->find(mid) << endl;
		return( 0L );
	}
}

/**
 ** CxCacheMgr::expire(): Expire an entry in our cache table.  (This doesn't actually _remove_
 ** the cache entry.  It sets the cache entry's ttl timestamp to something which will cause it
 ** to be refreshed upon next update...)
 **/
void		CxCacheMgr::expire( const QString&, long unsigned int ) {

	kdDebug() << "CxCacheMgr::expire(): " << endl;
}

/**
 ** CxCacheMgr::index(): Return a QDictIterator for the specified room, or null otherwise.
 **/
QDictIterator<QMesgInfo>
		CxCacheMgr::index( const QString& room ) {

	kdDebug() << "CxCacheMgr::index(): Return index for "<<room<<endl;
	if( g_RoomCache.find( room ) ) {

		kdDebug() << "CxCacheMgr::index(): Index found." << endl;
		QDictIterator<QMesgInfo> t( *(g_RoomCache.find( room )) );
		return(t);

	} else {

		kdDebug() << "CxCacheMgr::index(): Index not found." << endl;
		QDict< QMesgInfo > t;
		QDictIterator< QMesgInfo > x(t);
		return( x );

	}
}

/**
 ** CxCacheMgr::_insertRoom(): Allocate space for a new room in the
 ** cache table.  This may involve removing the least-frequently-accessed
 ** room...
 **/
bool		CxCacheMgr::_insertRoom( const QString& room ) {

	kdDebug() << "CxCacheMgr::_insertRoom(): ALLOCATE " << room << endl;

	/**
	 ** If our room exists in the cache table, let's not add it
	 ** again...
	 **/
	if( g_RoomCache[ room ] ) {
		kdDebug() << "CxCacheMgr::_insertRoom(): Exists." << endl;
		return( false );
	}

	QDict<QMesgInfo> *tmp = new QDict<QMesgInfo>;

	g_RoomCache.insert( room, tmp );

	time_t *t;
	t = new time_t;
	time( t );

	g_RoomCacheTtl.insert( room, t );

	kdDebug() << "CxCacheMgr::_insertRoom(): Inserted." << endl;
	return( true );
}

/**
 ** CxCacheMgr::_insertMessage(): Insert/update a message in the
 ** cache table.
 **/
bool		CxCacheMgr::_insertMessage( const QString& room, long unsigned int msg_id, QMesgInfo *minf ) {
long unsigned int flags = MSG_INIT;
char		buf[255];
 
	kdDebug() << "CxCacheMgr::_insertMessage(): Received request to insert message into cache." << endl;
	kdDebug() << "CxCacheMgr::_insertMessage(): GMID: " << room << ":" << msg_id << endl;
	if( room.isNull() || !msg_id ) return( false );

	sprintf( buf, "%ld", msg_id );

/*	if( minf ) {
		kdDebug() << "CxCacheMgr::_insertMessage(): MESSAGE INFO:" << endl;
		kdDebug() << "[MessageId: " << minf->MessageId << "]" << endl;
		kdDebug() << "minf->Author: " << minf->Author << endl;
		kdDebug() << "minf->Recipient: " << minf->Recipient << endl;
		kdDebug() << "minf->Date: " << minf->Date << endl;
		kdDebug() << "minf->Subject: " << minf->Subject << endl;
		kdDebug() << "minf->flags: " << minf->flags << endl;
		kdDebug() << "CxCacheMgr::_insertMessage(): /MESSAGE INFO:" << endl; 
	} */
 
	if(!minf->flags) {
		if( !minf->MessageBody.isNull() ) flags |= MSG_BODY;
		if( !minf->Author.isNull() && !minf->Date.isNull()) flags |= MSG_HEADER;
 
	} else {
		flags = minf->flags;
	}
 
	/**
	 ** If this message is already stored in our database table, check to see
	 ** if it is unchanged from this copy.  (generally, if flags != local flags,
	 ** it's changed...
	 **/
	if( g_RoomCache.find( room ) && (g_RoomCache.find( room ))->find( buf ) &&
		((g_RoomCache.find( room )->find(buf))->flags) == (flags) ) {
 
		kdDebug() << "CxCacheMgr::_insertMessage(): TIMESTAMP: " << minf->unix_timestamp << endl;
		kdDebug() << "CxCacheMgr::_insertMessage(): "<< flags << " == " <<
			(g_RoomCache.find( room )->find(buf))->flags << endl;

		kdDebug() << "CxCacheMgr::_insertMessage(): xxx No changes, not inserting." << endl;
		return( false );
 
	} else if( g_RoomCache.find( room ) && (g_RoomCache.find( room ))->find( buf ) ) {

		kdDebug() << "CxCacheMgr::_insertMessage(): "<< flags << " != " <<
			(g_RoomCache.find( room )->find(buf))->flags << endl;;
		kdDebug() << "CxCacheMgr::_insertMessage(): ~~~ Change in content.  Refreshing cache entry." << endl;

		kdDebug() << "CxCacheMgr::_insertMessage(): Remove entry @" << (g_RoomCache.find( room ))->find( buf ) << endl;

		QMesgInfo *nu = new QMesgInfo;
		memcpy( &nu, &minf, sizeof(nu) );
		nu->unix_timestamp = ((g_RoomCache.find( room ))->find( buf ))->unix_timestamp;

		(g_RoomCache.find( room ))->remove( QString(buf) );
		(g_RoomCache.find( room ))->insert( QString(buf), nu );
		kdDebug() << "CxCacheMgr::_insertMessage(): Add entry @" << (g_RoomCache.find( room ))->find( buf ) << endl;
		kdDebug() << "CxCacheMgr::_insertMessage(): (unix_timestamp: " << (unsigned long int)(((g_RoomCache.find( room ))->find( buf ))->unix_timestamp) << ")" << endl;

		return( true );
 
	} else {
		kdDebug() << "CxCacheMgr::_insertMessage(): +++ Inserting new content into cache." << endl;
		g_RoomCache.find( room )->insert( QString(buf), minf );
		return( true );
	}
}

/**
 ** CxCacheMgr::internalTableHndl(): Return a pointer to the table handle
 ** for the specified room.
 **/
QDict<QMesgInfo>	
		*CxCacheMgr::internalTableHndl( const QString& room ) {

	return( g_RoomCache.find( room ) );
}

/**
 ** CxCacheMgr::_roomExists(): Does the specifed room exist?
 **/
bool		CxCacheMgr::_roomExists( const QString& room ) {

	kdDebug() << "CxCacheMgr::_roomExists(): Room: \"" << room << "\"" << endl;
	return( ( g_RoomCache.find( room ) != NULL ) );
}

/**
 ** CxCacheMgr::_fresh(): Is this room fresh?
 **/
bool		CxCacheMgr::_roomFresh( const QString& room ) {

	kdDebug() << "CxCacheMgr::_roomFresh(): Room: \"" << room << "\"" << endl;
	return( _roomExists( room ) );
}

/**
 ** CxCacheMgr::_messageExists(): Does the specifed message exist?
 **/
bool		CxCacheMgr::_messageExists( const QString& room, long unsigned int msg_id ) {
char		buf[255];

	kdDebug() << "CxCacheMgr::_messageExists(): GUID: \"" << room << ":" << msg_id << "\"" << endl;
	sprintf( buf, "%ld", msg_id );
	return( (
			( g_RoomCache.find( room ) != NULL ) && 
			( (g_RoomCache.find( room ))->find( buf ) != NULL )
		)
	);
}

/**
 ** CxCacheMgr::_fresh(): Is the specified message fresh?
 **/
bool		CxCacheMgr::_messageFresh( const QString &room, long unsigned int msg_id ) {

	kdDebug() << "CxCacheMgr::_messageFresh(): GUID: \"" << room << ":" << msg_id << "\"" << endl;
	return( _messageExists( room, msg_id ) );
}

/**
 ** CxCacheMgr::internalCacheMaintenance(): Perform routine maintenance on our cache
 ** entries.  This slot should be called periodically by a QTimer...
 **/
void		CxCacheMgr::_cacheMaintenance() {

	kdDebug() << "CxCacheMgr::_cacheMaintenance(): Performing cache maintenance now..." << endl;
	kdDebug() << "CxCacheMgr::_cacheMaintenance(): (stub)" << endl;
}

