#ifndef		__CXCACHEMGR_H__
#define		__CXCACHEMGR_H__
#include	<qobject.h>
#include	<qdict.h>
#include	<qlist.h>
#include	<time.h>
#include	<dcopclient.h>
#include	<qstring.h>
#include	"cxconnection.h"

/**
 ** class CxCacheMgr: Generic cache management object.  This is intended
 ** to be bound to a parent CxConnection object, to which it will provide
 ** a folder/message/information caching mechanism.  This object will have
 ** no idea about network interfaces, but may use the parent CxConnection 
 ** object to obtain data from the network if it needs to.
 **/
class		CxCacheMgr: public QObject {
Q_OBJECT

public:
		CxCacheMgr( CxConnection *parent );
		~CxCacheMgr();

	/**
	 ** Generic cache accessor methods.
	 **/
	bool		exists( const QString&, long unsigned int = 0 );
	bool		isComplete( const QString&, long unsigned int );
	bool		isFresh( const QString&, long unsigned int = 0 );
	bool		insert( const QString&, long unsigned int = 0, QMesgInfo * = 0L );
	QMesgInfo	*fetch( const QString&, long unsigned int, unsigned int );
	void		expire( const QString&, long unsigned int );
	QDictIterator<QMesgInfo>
			index( const QString& );

private slots:
	void		_cacheMaintenance();

private:
	bool		_insertRoom( const QString& );
	bool		_insertMessage( const QString&, long unsigned int, QMesgInfo * );

	bool		_roomExists( const QString& );
	bool		_messageExists( const QString&, long unsigned int );

	bool		_roomFresh( const QString& );
	bool		_messageFresh( const QString&, long unsigned int );

	QDict<QMesgInfo>	
			*internalTableHndl( const QString& );

protected:

	/**
	 ** This is a table in which we can store cached room indices
	 ** on any room that we've visited (assuming that user doesn't
	 ** request the no_cache method).  This table will get huge
	 ** with time, so some heavy maintenance is required for this
	 ** (I'd have to keep some sort of method of tracking time
	 ** in the g_RoomCacheTtl table..  At a specified timeout
	 ** [arbitrary], we need to treat the cached table as DIRTY
	 ** and refresh it from the server.  We will keep a maximum
	 ** of CACHE_MAX rooms in cache.  If we get a request for a
	 ** new room to cache, we will discard the oldest room that
	 ** is currently in cache.)
	 **
	 ** Hope that isn't too confusing... ;)
	 **/
	QDict< time_t > g_RoomCacheTtl;
	QDict< QDict< QMesgInfo > >
		g_RoomCache;

	CxConnection	*g_Connection;
	QTimer		*maintTimer;
};
 
/**
 ** Global constants:
 **
 ** CACHE_TTL: This is the amount of time that we will keep a
 ** cached entry in g_RoomCache before we consider it to be
 ** "dirty" and in need of a refresh.
 **
 ** CACHE_MAX: This is the maximum number of rooms to keep in
 ** our cache table.  There needs to be a formula for expiring
 ** the most out-of-date, least frequently accessed rooms...
 **
 ** CACHE_ENTRY_TTL: This is the amount of time a cachetbl entry
 ** is expected to live in the database before being purged (or
 ** at the least, being considered "dirty").
 **/
#define		CACHE_TTL		3100
#define		CACHE_ENTRY_TTL		3100
#define		CACHE_MAX		20

#endif
