/**
 ** Infusion - The Desktop Communications Revoution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: cxconnection.o
 ** Date: 2001-08-23
 ** Last Revision: 2001-08-24
 ** Description: Citadel/UX Connection object.
 ** CVS: $Id: cxconnection.cpp,v 1.12 2001/08/28 01:20:58 brian Exp $
 **/
#include	<stdio.h>
#include	<stdlib.h>
#include	<qtimer.h>
#include	<qobject.h>
#include	<qlist.h>
#include	<qstring.h>
#include	<qdict.h>
#include	<sys/time.h>
#include	<iostream.h>
#include	<dcopclient.h>
#include	<kdebug.h>
#include	<CxClient.h>
#include	"cxconnection.h"
#include	"cxcachemgr.h"
#include	"autoconf.h"

/**
 ** CxConnection::CxConnection(): 
 **/		
		CxConnection::CxConnection( const QString& host, const QString& user, const QString& pass )
		:QObject( 0, "CxConnection" ) {

	kdDebug() << "CxConnection::CxConnection(): *** Creating new Citadel/UX connection" << endl;
	kdDebug() << "CxConnection::CxConnection(): @" << this << endl;
	g_userName = new QString( user );
	g_password = new QString( pass );
	g_serverName = new QString( host );
	g_isOnline = false;
	g_isConfigured = false;
	g_hasMail = false;

	l_floorIndex = l_roomsIndex = l_usersIndex = l_allUsersIndex = 
		l_messageIndex = l_messageInfoIndex = l_messageUnreadIndex = 0L;
	g_currentRoom = QString::null;

	g_CacheMgr = new CxCacheMgr( this );
}


/**
 ** CxConnection::CxConnection(): 
 **/		
		CxConnection::~CxConnection() {

	if(g_isOnline) disconnect();

	if(g_cxHndl) CxClDelete(g_cxHndl);

	delete g_CacheMgr;

	delete g_userName;
	delete g_password;
	delete g_serverName;
}

/**
 ** CxConnection::connect(): 
 **/
void		CxConnection::connect() {
USERINFO	*info;
int		myerr;

	if(g_isOnline || !g_isConfigured) return;
	if(g_serverName->isEmpty()) return;
	if(g_userName->isEmpty()) return;

	g_cxHndl = CxClConnection( g_serverName->ascii(), 504, g_userName->ascii(), g_password->ascii() );
	if(!g_cxHndl) return;

	/**
	 ** Connect to the Citadel/UX host.
	 **/
	myerr = CxClConnect( g_cxHndl );
	if(!myerr) {
		emit comingOnline();

		if(  	( g_userName && (!g_userName->isNull())) &&
			( g_password ) ) {

			if(!(info = CxUsAuth( g_cxHndl, NULL, NULL ))) {  //g_userName->ascii(), g_password->ascii()))) {
				disconnect();

				emit badPassword(g_userName->copy(), g_serverName->copy());
				emit amOffline();

			} else {
				free(info);
				g_isOnline = true;

				QTimer *refreshTimer = new QTimer(this);
				refreshTimer->changeInterval( 2500 );
				QObject::connect(refreshTimer, SIGNAL(timeout()), this, SLOT(checkExpressMsg()));

				emit amOnline();

			}
		}

	} else {
		emit error(QString("Unable to connect to Citadel/UX server!\n")+QString(strerror(abs(myerr))));
	}
}

/**
 ** CxConnection::disconnect(): 
 **/
void		CxConnection::disconnect() {

	if(!g_isOnline) return;

	CxClDisconnect( g_cxHndl );
	CxClDelete( g_cxHndl );
	g_cxHndl = 0;
	g_isOnline = false;

	emit goingOffline();
	emit amOffline();
}

/**
 ** CxConnection::sync(): Synchronize my cached indices with the indices on the server.
 **/
void		CxConnection::sync() {

	kdDebug() << "CxConnection::sync()" << endl;
	emit updatingStart();

	QTimer::singleShot( 500, this, SLOT(syncSlave()));
}

/**
 ** CxConnection::syncSlave(): This is the REAL synchronization handler.
 **/
void		CxConnection::syncSlave() {

	kdDebug() << "CxConnection::syncSlave()" << endl;
	emit syncFloors();
	emit syncRooms();
	emit syncUsers();
	QTimer::singleShot( 500, this, SIGNAL(updatingFinish()));
}

/**
 ** CxConnection::syncFloors(): Synchronize floors index.
 **/
void		CxConnection::syncFloors() {

	kdDebug() << "CxConnection::syncFloors()" << endl;
	if(l_floorIndex) l_floorIndex = CxLlFlush( l_floorIndex );
	l_floorIndex = CxFlList( g_cxHndl );
}

/**
 ** CxConnection::syncRooms(): Synchronize rooms index.
 **/
void		CxConnection::syncRooms() {

	kdDebug() << "CxConnection::syncRooms()" << endl;
	if(l_roomsIndex) l_roomsIndex = CxLlFlush( l_roomsIndex );
	l_roomsIndex = CxRmList( g_cxHndl );
}

/**
 ** CxConnection::syncUsers(): Synchronize users index.
 **/
void		CxConnection::syncUsers() {

	kdDebug() << "CxConnection::syncUsers()" << endl;
	if(l_usersIndex) l_usersIndex = CxLlFlush( l_usersIndex );
	l_usersIndex = CxUsOnline(  g_cxHndl, 0 );

	if(!l_allUsersIndex) {
		l_allUsersIndex = CxUsList( g_cxHndl );
	}
}

/**
 ** CxConnection::getOnlineUsers(): Return a QList vector of online users.
 **/
QList<QString>	CxConnection::getOnlineUsers() {
CXLIST		x;
QList<QString>	toret;

	kdDebug() << "CxConnection::getOnlineUsers()" << endl;
	if(!l_usersIndex) return(toret);

	x = l_usersIndex;
	while( x ) {
		QString *foo = new QString( x->data );
		toret.append( foo );
		x = x->next;
	}

	return( toret );
}

/**
 ** CxConnection::getAllUsers(): Return a QList vector of all system users.
 **/
QList<QString>	CxConnection::getAllUsers() {
CXLIST		x;
QList<QString>	toret;

	kdDebug() << "CxConnection::getAllUsers()" << endl;
	if(!l_allUsersIndex) return(toret);

	x = l_allUsersIndex;
	while( x ) {
		QString *foo = new QString( x->data );
		toret.append( foo );
		x = x->next;
	}

	return( toret );
}

/**
 ** CxConnection::sendExpressMsg(): Send an express message to a remote user.
 **/
void		CxConnection::sendExpressMsg( const QString& user, const QString& message ) {

	kdDebug() << "CxConnection::sendExpressMsg()" << endl;
	if( CxMiExpSend( g_cxHndl, user.ascii(), message.ascii() ) ) {
		emit error(QString("Unable to send Express Message to ")+user);
	}
}

/**
 ** CxConnection::recvExpressMsg(): Receive the pending express message.
 **/
void		CxConnection::recvExpressMsg() {

	kdDebug() << "CxConnection::recvExpressMsg()" << endl;

	EXPRMESG *s = CxMiExpRecv( g_cxHndl );

	if( s ) {
		kdDebug() << "EXPRMESG" << endl;
		kdDebug() << ".more_follows: " << s->more_follows << endl;
		kdDebug() << ".timestamp: " << s->timestamp << endl;
		kdDebug() << ".flags: " << s->flags << endl;
		kdDebug() << ".sender: " << s->sender << endl;
		kdDebug() << ".node: " << s->node << endl;
		kdDebug() << ".message: " << s->message << endl;
	} else {
		kdDebug() << "!EXPRMESG" << endl;
	}
 
	if(!s) {
		return;
	}
 
	emit handleExpressMsg( QString(s->sender), QString(s->message) );

/**
	CxWndChat *win = &chatWin(QString(g_Ser[3]));
	if(win) {
		win->recvMesg(g_Ser[5]);
		win->show();
	}
 **/

	free(s->message);
	free(s);
}

/**
 ** CxConnection::checkExpressMsg(): Check for express messages.
 **/
void		CxConnection::checkExpressMsg() {

	if(CxMiExpCheck( g_cxHndl )) {
		emit recvExpressMsg();
	}
}

/**
 ** CxConnection::floorCache(): Return a reference of the floor cache.
 **/
CXLIST		*CxConnection::floorCache() {

	return( &l_floorIndex );
}

/**
 ** CxConnection::checkMail(): Check the Citadel/UX host for new mail.
 **/
void		CxConnection::checkMail() {
CXLIST		mlst = 0L;
ROOMINFO	*info;

	kdDebug() << "CxConnection::checkMail()" << endl;
	info = CxRmGoto(g_cxHndl, "_MAIL_", 0);
	free( info );

	mlst = CxMsList( g_cxHndl, MSGS_NEW, 0 );
	if( mlst ) {
		kdDebug() << "CxConnection::checkMail(): * NEW MAIL *" << endl;
		mlst = CxLlFlush( mlst );
		g_hasMail = true;
		emit hasMail();
	}
}

/**
 ** CxConnection::fetchFolderList(): Fetch a list of folders on this
 ** connection.  On Citadel/UX, this will be (essentially) a merge of
 ** the Floor and Room listings...
 **/
QCStringList	CxConnection::fetchFolderList() {
QCStringList	results;
QDict<QString>	floorIndex;
CXLIST		x;
char		buf[1024];

	if(!g_isOnline) return( results );

	/**
	 ** IF our data points aren't up to date, then we
	 ** need to sync them with the server...  (but this
	 ** should be a little bit more flexible...)
	 **/
	if( !l_floorIndex ) emit syncFloors();
	if( !l_roomsIndex ) emit syncRooms();

	x = l_floorIndex;
	while( x ) {
		strcpy( buf, x->data );
		CxSerialize( buf, (char **) &g_Ser );

		floorIndex.insert( QString(g_Ser[0]), new QString(x->data));
		x = x->next;
	}

	x = l_roomsIndex;
	while( x ) {
		strcpy( buf, x->data );
		CxSerialize( buf, (char **) &g_Ser );

		QCString data( (*floorIndex[QString(g_Ser[2])]) + QString("|") + x->data );
		results.append( data );

		x = x->next;
	}

	floorIndex.clear();

	return( results );
}

/**
 ** CxConnection::fetchNotes(): Fetch the notes from the server.  This
 ** is essentially a FetchMessageList() call, without updating the message
 ** indices or moving the currentRoom pointer.  This should eventually be
 ** moved to some more generic fetchMessages() function....
 **/
QCStringList	CxConnection::fetchNotes() {
QCStringList	results;

	kdDebug() << "CxConnection::fetchNotes(): I really think this function should go away..." << endl;
	QCStringList roomIndex = fetchMessageList( QString("My Notes") );
	QCStringList::Iterator x;

	for( x = roomIndex.begin(); x != roomIndex.end(); ++x ) {
		QString y = (*x).mid(0, (*x).find('|'));

		QMesgInfo *m = fetchMesgInfo( QString("My Notes"), y.toULong(), true );
		if( m ) {
			kdDebug() << "CxConnection::fetchNotes(): Loaded message @" << m << endl;
			results << m->MessageBody;
		}
/**
		QMesgInfo *m = g_CacheMgr->fetch( QString("My Notes"), y.toULong(), 0L );
		if( m ) {
			kdDebug() << "CxConnection::fetchNotes(): Loaded message @" << m << endl;
			if((m->flags & MSG_BODY) == MSG_BODY) {
				kdDebug() << "CxConnection::fetchNotes(): Complete message." << endl;
				results << m->MessageBody;
			} else {
				m = fetchMesgInfo( QString("My Notes"), y.toULong, false );
				if(m && (m->flags & MSG_BODY) == MSG_BODY) {
					g_CacheMgr->insert( QString("My Notes"), y.toULong(), m );
					results << m->MessageBody;
				} else {
				}
			}
		} 
 **/
	}
	return( results );
}

/**
 ** CxConnection::fetchMessageList(): Fetch a list of messages for the
 ** specified room.  Store the results in our object's cache table if
 ** we with to look again later.
 **/
QCStringList	CxConnection::fetchMessageList( const QString& room, bool cache_results ) {
char		*ser[20], buf[1024];
ROOMINFO	*info;
CXLIST		x, y, z;
CXLIST		messageIndex = 0L,
		messageInfoIndex = 0L,
		messageUnreadIndex = 0L;
QString		local_room;
QCStringList	res;
int		new_message;
bool		b;

	if(!g_isOnline) return( res );
	if( room.isEmpty() ) return( res );

	local_room = resolveRoomName( room );
	kdDebug() << "CxConnection::fetchMessageList(): room: " << room << endl;
	kdDebug() << "CxConnection::fetchMessageList(): local_room: " << local_room << endl;

	/**
	 ** Jump to the specified room.
	 **/
	if( local_room != g_currentRoom ) {
		info = CxRmGoto( g_cxHndl, local_room.ascii(), 0 );
		free( info );
	}

	/**
	 ** IF this room is in our g_RoomCache, AND the Ttl on the room
	 ** isn't out-of-bounds...
	 **/
	if( (b = g_CacheMgr->exists( local_room )) && g_CacheMgr->isFresh( local_room ) ) {

		/**
		 ** Obtain our data from our g_RoomCache and
		 ** return it, rather than bugging the Citadel/UX
		 ** server...
		 **/
		kdDebug() << "CxConnection::fetchMessageList(): Cache is clean.  Populating results..." << endl;

		QDictIterator<QMesgInfo> tmp = g_CacheMgr->index( local_room );
		if( tmp.count() > 0 ) {

			while( tmp.current() ) {
				kdDebug() << "CxConnection::fetchMessageList(): " << tmp.currentKey() << endl;

				QCString date;
				kdDebug() << "**** NUMERIC TIMESTAMP: " << (long unsigned int) ((tmp.current())->unix_timestamp) << endl;
				date.setNum( (unsigned long int) ((tmp.current())->unix_timestamp));
				kdDebug() << "**** TEXTUAL TIMESTAMP: " << date << endl;

				res.append(
					QCString( tmp.currentKey() ) + QCString("|") +
					QCString( "0" ) + QCString("|") +
					QCString( (tmp.current())->Author ) + QCString("|") +
					QCString( (tmp.current())->Recipient ) + QCString("|") +
					QCString( "" ) + QCString("|") +
					date/*QCString( (tmp.current())->Date )*/ + QCString("|") +
					QCString( (tmp.current())->Subject ) + QCString("|")
				);

				++tmp;
			}

			//delete(tmp);

			kdDebug() << "CxConnection::fetchMessageList(): Completed, returning!" << endl;
			return( res );
		}

	/**
	 ** OTHERWISE, if the local_room exists in cache, we need to refresh
	 ** the entries in the table...
	 **/
	} else if( b ) {

		kdDebug() << "CxConnection::fetchMessageList(): Cache needs refresh." << endl;

	/**
	 ** OTHERWISE, it's time to insert this local_room into our cache
	 ** table.
	 **/
	} else {

		kdDebug() << "CxConnection::fetchMessageList(): Begin cache population..." << endl;

		/**
		 ** Prepare a place in our table...
		 **/
		if(g_CacheMgr->insert( local_room )) {
	
			messageIndex = CxMsList( g_cxHndl, MSGS_ALL, 0 );
			messageInfoIndex = CxMsInfo( g_cxHndl, messageIndex );
			messageUnreadIndex = CxMsList( g_cxHndl, MSGS_NEW, 0 );

			CXLIST x = messageIndex;
			CXLIST y = messageInfoIndex;
			while( x && y ) {

				z = messageUnreadIndex;
				new_message = 0;
				while( z ) {
					if(!strcmp(y->data, z->data)) {
						new_message = 1;
						break;
					}
					z = z->next;
				}

				long unsigned int msg_id = (long unsigned int) atol( x->data );
				QMesgInfo *msg = new QMesgInfo;

				strcpy( buf, y->data );
				CxSerialize( buf, (char **) &ser );
				msg->MessageId = msg_id;
				msg->Author = QCString( ser[0] );
				msg->Date = QCString( ser[1] );
				msg->Subject = QCString( ser[2] );
				msg->flags = MSG_INIT | (new_message?MSG_UNREAD:0);
				kdDebug() << "**** TEXTUAL TIMESTAMP: " << ser[1] << endl;
				msg->unix_timestamp = strtoul( ser[1], 0L, 10 );
				kdDebug() << "**** NUMERIC TIMESTAMP: " << msg->unix_timestamp << endl;

				g_CacheMgr->insert( local_room, msg_id, msg );

				msg->Date.setNum(msg->unix_timestamp);

				res.append(
					QCString( x->data ) + QCString("|") +
					QCString( "0" ) + QCString("|") +
					QCString( msg->Author ) + QCString("|") +
					QCString( msg->Recipient ) + QCString("|") +
					QCString( "" ) + QCString("|") +
					QCString( msg->Date ) + QCString("|") +
					QCString( msg->Subject ) + QCString("|")
				);
/**
 **				res.append( QCString(x->data)+QCString("|")+
 **					(new_message?QCString("1"):QCString("0"))+QCString("|")+
 **					QCString( y->data ));
 **/

				x = x->next;
				y = y->next;
			}

			messageIndex = CxLlFlush( messageIndex );
			messageInfoIndex = CxLlFlush( messageInfoIndex );
			messageUnreadIndex = CxLlFlush( messageUnreadIndex );

			return( res );

		/**
		 ** FAILED!
		 **/
		} else {
			kdDebug() << "CxConnection::fetchMessageList(): Unable to allocate cache table space." << endl;
			return( res );
		}
	}

	/**
	 ** If we've never been to this room, or our message caches hav
	 ** become corrupted, we need to refresh our caches.
	 **/
	if( (local_room != g_currentRoom) || (!l_messageIndex && !l_messageInfoIndex) ) {

		/**
	 	 ** Rebuild our message info caches.
		 **/
		messageIndex = CxMsList( g_cxHndl, MSGS_ALL, 0 );
		messageInfoIndex = CxMsInfo( g_cxHndl, messageIndex );
		messageUnreadIndex = CxMsList( g_cxHndl, MSGS_NEW, 0 );

	} else {
		messageIndex = l_messageIndex;
		messageInfoIndex = l_messageInfoIndex;
		messageUnreadIndex = l_messageUnreadIndex;
	}

	/**
	 ** If we have data...
	 **/
	if( messageIndex && messageInfoIndex ) {

		/**
		 ** Walk through and build our array...
		 **/
		x = messageIndex;
		y = messageInfoIndex;

		while( x && y ) {

			z = messageUnreadIndex;
			new_message = 0;
			while( z ) {
				if(!strcmp(y->data, z->data)) new_message = 1;
				z = z->next;
			}
			res.append( QCString(x->data)+QCString("|")+
					(new_message?QCString("1"):QCString("0"))+QCString("|")+
					QCString( y->data ));

			x = x->next;
			y = y->next;
		}
	}

	/**
	 ** Do we wish to cache the results of this transaction,
	 ** for later referencing?
	 **/
	if( cache_results && (local_room != g_currentRoom) ) {

		/**
		 ** Clear our message info caches.
		 **/
		l_messageIndex = CxLlFlush( l_messageIndex );
		l_messageInfoIndex = CxLlFlush( l_messageInfoIndex );
		l_messageUnreadIndex = CxLlFlush( l_messageUnreadIndex );

		l_messageIndex = messageIndex;
		l_messageInfoIndex = messageInfoIndex;
		l_messageUnreadIndex = messageUnreadIndex;

		g_currentRoom = local_room;

	} else if(!cache_results) {
		info = CxRmGoto( g_cxHndl, g_currentRoom.ascii(), 0 );
		free( info );
		messageIndex = CxLlFlush( messageIndex );
		messageInfoIndex = CxLlFlush( messageInfoIndex );
		messageUnreadIndex = CxLlFlush( messageUnreadIndex );
	}

	return( res );
}

/**
 ** CxConnection::deliverMessage(): Attempt the message
 ** delivery specified by the QCStringList.  Fields
 ** will be defined below.  Return 0 on success, nonzero 
 ** on failure.
 **/
int		CxConnection::deliverMessage( QCStringList *sl ) {
int		i;
ROOMINFO	*info;
QCStringList::Iterator x;
int		results;
QCString	room,
		from,
		to,
		cc,
		subject,
		body;

	if(!g_isOnline) return( -1 );

	i = 0;
	for(x = sl->begin(); x != sl->end(); ++x) {
		switch(i) {
			case(0): room = (*x); break;
			case(1): break; /** We don't use this field here. **/
			case(2): from = (*x); break;
			case(3): to = (*x); break;
			case(4): cc = (*x); break;
			case(5): subject = (*x); break;
			case(6): body = (*x); break;
			default: kdDebug() << "excessive data!" << endl; break;
		}
		++i;
	}

	/**
	 ** Do we have the correct number of datums?  Do we have the
	 ** bare minimum of delivery data?
	 **/
	if( (i == 7) && (!room.isNull() && !to.isNull()) ) {
		MESGINFO msg;
		msg.body = 0L;

		strcpy(msg.author, from);
		strcpy(msg.rcpt, to);
		strcpy(msg.subject, subject);
		msg.body = (char *)malloc(strlen(body)+1);
		strcpy( msg.body, body );

		info = CxRmGoto( g_cxHndl, room, 0 );
		free( info );

		/**
		 ** Attempt to save the message...
		 **/
		if(CxMsSave(g_cxHndl, msg)) {
			results = -2;

		} else {
			results = 0;
		}

		free( msg.body );
	} else results = -1;

	info = CxRmGoto( g_cxHndl, g_currentRoom.ascii(), 0 );
	free( info );

	return( results );
}

/**
 ** CxConnection::fetchMesgInfo(): Retrieve a message from the server.  Return the message
 ** encapsulated in a QMesgInfo struct.
 **/
QMesgInfo	*CxConnection::fetchMesgInfo( const QString &room, long unsigned int msg_id, bool try_cache ) {
QMesgInfo	*ret = 0L;
ROOMINFO	*info;
MESGINFO	minf;

	kdDebug() << "CxConnection::fetchMesgInfo(): --------------------------------------------------------------" << endl;
	kdDebug() << "CxConnection::fetchMesgInfo(): FETCH " << room << ":" << msg_id << endl;
	kdDebug() << "CxConnection::fetchMesgInfo(): try_cache: " << try_cache << endl;

	if(!g_isOnline) return( 0L );

	/**
	 ** Abort if we have insufficient data.  [Question: If room==null,
	 ** should we assume that they meant "CURRENT" room?]
	 **/
	if( room.isEmpty() || !msg_id ) return( 0L );

	/**
	 ** Try to load this message from our cache table.
	 **/
	if( try_cache ) {

		QMesgInfo *m = 0L;
		kdDebug() << "CxConnection::fetchMesgInfo(): Trying cache..." << endl;
		m = g_CacheMgr->fetch( room, msg_id, 0L);

		if( m ) {
			kdDebug() << "CxConnection::fetchMesgInfo(): m @" << m << endl;

			if( (m->flags&(MSG_BODY|MSG_HEADER))==(MSG_BODY|MSG_HEADER) ) {
				kdDebug() << "CxConnection::fetchMesgInfo(): Message complete.  Returning..." << endl;
				return( m );
			} else {
				kdDebug() << "CxConnection::fetchMesgInfo(): Message incomplete.  Need to refresh." << endl;
				kdDebug() << "CxConnection::fetchMesgInfo(): m->flags: " << m->flags << endl;
				delete m;
			}
		} else {
			kdDebug() << "CxConnection::fetchMesgInfo(): Message not in cache.  Need to refresh." << endl;
		}
	}

	/**
	 ** Step 1: Goto target room.  Otherwise, fail.
	 **/
	kdDebug() << "CxConnection::fetchMesgInfo(): Need to fetch complete message from server." << endl;
	if((info = CxRmGoto( g_cxHndl, room, 0 ))) {
		free(info);
		minf.body = 0L;

		int i;
		if(!(i = CxMsLoad( g_cxHndl, msg_id, 1, &minf ))) {
			kdDebug() << "CxConnection::fetchMesgInfo(): Message retrieved.  Populating QMesgInfo." <<endl;

			ret = new QMesgInfo;
			ret->MessageId = msg_id;
			ret->Date = minf.date;			
			ret->Author = minf.author;			
			ret->Recipient = minf.rcpt;			
			ret->Subject = minf.subject;			
			ret->MessageBody = minf.body;			
			ret->flags = (MSG_HEADER|MSG_BODY);
			free(minf.body);

			kdDebug() << "CxConnection::fetchMesgInfo(): Populating cache..." << endl;
			g_CacheMgr->insert( room, msg_id, ret );

			kdDebug() << "CxConnection::fetchMesgInfo(): Returning" << endl;
			return( ret );
		} else {
			kdDebug() << "CxConnection::fetchMesgInfo(): CxMsLoad() returned " << i << endl;
		}

		info = CxRmGoto( g_cxHndl, g_currentRoom.ascii(), 0 );
		free(info);
	} else {
		kdDebug() << "CxConnection::fetchMesgInfo(): No room " << room << endl;
	}

	/**
	 ** Return whatever we have.
	 **/
	return( 0L );
}

/**
 ** CxConnection::fetchMessage(): Retrieve a message from the server.  Return the message
 ** in a DCOP-ready stringlist.  (This calls the above function.)
 **/
QCStringList	CxConnection::fetchMessage( const QCString &room, long unsigned int msg_id ) {
QString		local_room = resolveRoomName( room );
QMesgInfo	*msg;
QCStringList	results;

	kdDebug() << "CxConnection::fetchMessage(): GUID: \"" << room << ":" << msg_id << "\"" << endl;
	msg = fetchMesgInfo( local_room, msg_id, true );

	/**
	 ** If we loaded an actual message...
	 **/
	if( msg ) {
		results << QCString(local_room);
		results << msg->Date;
		results << msg->Author;
		results << msg->Recipient;
		results << QCString(""); // msg.Cc;
		results << msg->Subject;
		results << msg->MessageBody;
		return( results );

	/**
	 ** Otherwise, return nothing.
	 **/
	} else {
		return( results );
	}
}

/**
 ** CxConnection::resolveRoomName(): Attempt to translate some of
 ** the standard room names.
 **/
QString		CxConnection::resolveRoomName( const QString& room ) {
QString		local_room;

	/**
	 ** If this is called with a NULL argument, default to the
	 ** current room.  If there is no current room, default to
	 ** _MAIL_.
	 **/
	if(room == "Mail") local_room = QString("_MAIL_");
	else if(room.isNull() && !g_currentRoom.isNull()) local_room = g_currentRoom;
	else if(room.isNull() && g_currentRoom.isNull()) local_room = QString("_MAIL_");
	else local_room = room;

	return( local_room );
}

/**
 ** CxConnection::cacheStats(): Return a QCStringList of various cache
 ** statistics (mostly, size)
 **/
QCStringList	CxConnection::cacheStats() {
QCStringList	res;
QCString	x,y;

	res << "infusiond: The Infusion Daemon";
	res << "Copyright (c) 2000-2001, Flaming Sword Productions";
	res << "";
/*
	res << "Cache Statistics:";

	x.setNum( g_RoomCache.count() );
	y.setNum( CACHE_MAX );
	res << QCString("Number of Cached Rooms: ") + x + QCString(" of ") + y;

	res << "";
	res << "Room Index Counts:";
	QDictIterator< QDict<QMesgInfo> > qdi( g_RoomCache );

	while( qdi.current() ) {
		x.setNum((qdi.current())->count());
		res << QCString(qdi.currentKey()) + QCString(": ") + x;

		++qdi;
	}
 */

	res << "";
	res << "** END TRANSMISSION **";

	return(res);
}
