/**
 ** infusion - The Desktop Communications Revoution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: cxdaemon.o
 ** Date: 2001-03-08
 ** Last Revision: 2001-03-08
 ** Description: CxDaemon interface.
 ** CVS: $Id: cxdaemontray.cpp,v 1.14 2001/08/28 01:20:58 brian Exp $
 **/
#include	<unistd.h>
#include	<iostream.h>
#include	<qtimer.h>
#include	<qdatetime.h>
#include	<qstring.h>
#include	<qtooltip.h>
#include	<qdrawutil.h>
#include	<qbitmap.h>
#include	<kapp.h>
#include	<kdebug.h>
#include	<kpopupmenu.h>
#include	<kcmdlineargs.h>
#include	<kpixmapeffect.h>
#include	<kdebug.h>
#include	<kiconeffect.h>
#include	<klocale.h>
#include	<kglobal.h>
#include	<kconfig.h>
#include	<ksimpleconfig.h>
#include	<kstyle.h>
#include	<kiconloader.h>
#include	<kprocess.h>
#include	<kmessagebox.h>
#include	<knotifyclient.h>
#include	<kpassdlg.h>
#include	<CxClient.h>
#include	<autoconf.h>
#include	"cxdaemon.h"
#include	"wndabout.h"
#include	"wndbuddylist.h"
#include	"wndprefs.h"
#include	"cxconnection.h"
#include	"../icons/mail_new_16x16.xpm"

CxDaemonTray	*g_daemonTray = 0L;

/**
 ** CxDaemonTray::CxDaemonTray(): CONSTRUCTOR.
 **/
		CxDaemonTray::CxDaemonTray(QObject *parent, const char *name)
		:KSystemTray( NULL, name ) {

	if(parent) ;

	g_daemonTray = this;

	emit initIcons();
	emit setIconOffline();
	emit initConnection();
 
	g_newMailWaiting = false;
	g_savePassword = false;
	g_firstRun = false;

	itemId = contextMenu()->insertItem(i18n("&Online"), this, SLOT(toggleAlarmsEnabled()));
	contextMenu()->setItemChecked(itemId, FALSE);
	refreshId = contextMenu()->insertItem( i18n("&Refresh"), this, SLOT( syncLists() ));
	contextMenu()->insertItem(i18n("&Buddy List..."), this, SLOT(raiseBuddyDlg()));
	contextMenu()->setItemEnabled( refreshId, FALSE );
	contextMenu()->insertItem(i18n("&Preferences..."), this, SLOT(raisePrefsDlg()));
	contextMenu()->insertItem(i18n("&About..."), this, SLOT(raiseAboutDlg()));
	addToolTip( i18n("Offline") );

	connect( this, SIGNAL(goOnline()), this, SLOT(slotGoOnline()));
	connect( this, SIGNAL(goOffline()), this, SLOT(slotGoOffline()));

	w_buddyList = 0L;
	QTimer::singleShot( 230, this, SLOT(slotGoOnline()));
}

/**
 ** CxDaemonTray::
 **/
		CxDaemonTray::~CxDaemonTray() {

	delete g_cxConxion;
}

/**
 ** CxDaemonTray::initConnection(): Initialize the Citadel/UX connection object.
 **/
void		CxDaemonTray::initConnection() {
QString		tmp;
KConfig		*kc = kapp->config();

	g_cxConxion = new CxConnection;

	kc->setGroup("Accounts");
	tmp = kc->readEntry( "Configured" );
	if(!tmp.isNull()) {
		tmp = kc->readEntry( "Host" );
		if(!tmp.isNull()) g_cxConxion->setHostName( tmp );
		tmp = kc->readEntry( "Username" );
		if(!tmp.isNull()) g_cxConxion->setUserName( tmp );
		tmp = kc->readEntry( "Save Password" );
		if(tmp == "1") {
			g_savePassword = 1;
			tmp = kc->readEntry( "Password" );
			if(!tmp.isNull()) {
				g_savePassword = 0; // ?
				g_cxConxion->setPassword( tmp );
			}

		} else {
			g_savePassword = 0;
			g_cxConxion->setPassword( QString::null );
		}

	} else {
		emit raisePrefsDlg();
	}

	connect(g_cxConxion, SIGNAL(handleExpressMsg(const QString&, const QString&)), 
			this, SLOT(slotExpressMsg(const QString&, const QString&)));
	connect(g_cxConxion, SIGNAL(error(const QString&)), this, SLOT(errorMsg(const QString&)));
	connect(g_cxConxion, SIGNAL(badPassword(const QString&,const QString&)), this, SLOT(verifyPassword(const QString&,const QString&)));
	connect(g_cxConxion, SIGNAL(comingOnline()), this, SLOT(setIconWorking()));
	connect(g_cxConxion, SIGNAL(goingOffline()), this, SLOT(setIconOffline()));
	connect(g_cxConxion, SIGNAL(amOnline()), this, SLOT(setIconOnline()));
	connect(g_cxConxion, SIGNAL(updatingStart()), this, SLOT(setIconWorking()));
	connect(g_cxConxion, SIGNAL(updatingFinish()), this, SLOT(setIconNormal()));
	connect(g_cxConxion, SIGNAL(hasMail()), this, SLOT(setIconNewMail()));
	connect(this, SIGNAL(checkMail()), this, SLOT(doCheckMail()));
}

/**
 ** CxDaemonTray::verifyPassword(): Request the password for this account.
 **/
void		CxDaemonTray::verifyPassword( const QString& Account, const QString& Server ) {
KConfig		*kc = KGlobal::config();
int		result;
QCString	password;
QString		tmp;
const QString	prompt = i18n("Please enter the password for %1").arg(Account+QString("@")+Server);

	result = KPasswordDialog::getPassword(password, prompt);
	if( result == KPasswordDialog::Accepted ) {

		g_cxConxion->setPassword( password );

		/**
		 ** Save the password, if we're supposed to.
		 **/
		kc->reparseConfiguration();
		kc->setGroup("Accounts");
		tmp = kc->readEntry( "Save Password" );
		if(tmp == "1") {
			kc->writeEntry( QString("Password"), QString(password) );
			kc->sync();
		}

		emit goOnline();
	}
}

/**
 ** CxDaemonTray::getUserName(): Obtain the username for connection [x].  This will
 ** need to be modified in order to accomplish connection parallelization. [PARASPT]

}
 **/

/**
 ** CxDaemonTray::initIcons(): Load and initialize the icons.
 **/
void		CxDaemonTray::initIcons() {

	KGlobal::iconLoader()->addAppDir("infusion");
	pix1 = BarIcon(QString("infusion"));
	pix2 = BarIcon(QString("infusion"));
	pix3 = BarIcon(QString("connect_no"));
	pixNewMail = QPixmap( mail_new_16x16_xpm );

	QImage img = pix2.convertToImage();
	KIconEffect *mpEffect = new KIconEffect;
	img = mpEffect->apply( img, KIconEffect::NoEffect, 1.0, QColor(144,128,248), true );
	pix2.convertFromImage( img );

	if (pix1.isNull() || pix2.isNull()) {
		KMessageBox::sorry(this, i18n("Can't load docking tray icon!"),
				i18n("Alarm Monitor Error"));
	}

}

/**
 ** CxDaemonTray::syncLists(): Instruct the CxConnection object to refresh any and all lists
 ** that it is keeping track of.  This also flushes the miscellaneous data caches which exist
 ** for the convenience of the User.
 **/
void		CxDaemonTray::syncLists() {

	emit g_cxConxion->sync();
}

/**
 ** CxDaemonTray::autoSyncLists(): Automated hook for syncLists().  This should execute
 ** repeatedly to keep CxDaemon up-to-date.
 **/
void		CxDaemonTray::autoSyncLists() {

	/**
	 ** If we are NOT online, break this nasty cycle.
	 **/
	if(!g_cxConxion->isOnline()) return;

	emit syncLists();
	QTimer::singleShot( 100000, this, SLOT(autoSyncLists()));
}

/**
 ** CxDaemonTray::errorMsg(): Display an error dialog.  This generally means that the
 ** Infusion Daemon was unable to connect to the server, or some other extraneous event
 ** hath occurred.
 **/
void		CxDaemonTray::errorMsg( const QString& err ) {

	KMessageBox::sorry(this,
                                QString("An error has occurred:\n")+
                                err,
                                QString("Error"),
                                false
                        );
}

/**
 ** CxDaemonTray::
 **/
bool		CxDaemonTray::alarmsOn() {
	return( false );
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::addToolTip(const QString &msg) {

	QString txt = i18n("Infusion ")+i18n(VERSION);
	if (!msg.isEmpty()) txt += "\n" + msg;
	QToolTip::add(this,txt);
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::mousePressEvent(QMouseEvent *e) {

	if (e->button() == LeftButton) {

		emit launchInfusion();

	} else {
		KSystemTray::mousePressEvent(e);
	}
}

/**
 ** CxDaemonTray::launchInfusion(): Launch the master "Infusion" process.
 **/
void		CxDaemonTray::launchInfusion() {

	KProcess proc;
	proc << "infusion";
	proc.start(KProcess::DontCare);
}

/**
 ** CxDaemonTray::
 **/
void		CxDaemonTray::closeEvent(QCloseEvent *e) {

	if(e); 
	kapp->quit();
}

/**
 ** CxDaemonTray::toggleAlarmsEnabled():
 **/
void		CxDaemonTray::toggleAlarmsEnabled() {

	if(contextMenu()->isItemChecked(itemId)) {
		emit goOffline();
	} else {
		emit goOnline();
	}
}

/**
 ** CxDaemonTray::reapBuddyDlg(): Reap the Buddy List window.
 **/
void		CxDaemonTray::reapBuddyDlg() {

	w_buddyList = 0L;
}

/**
 ** CxDaemonTray::raiseBuddyDlg(): Open up the Buddy List window.
 **/
void		CxDaemonTray::raiseBuddyDlg() {
QList<QString>	list;

	if(!w_buddyList) {
		w_buddyList = new CxWndBuddylist();
		connect( w_buddyList, SIGNAL(closing()), this, SLOT(reapBuddyDlg()));
		connect( w_buddyList, SIGNAL(openChatWindow( const QString& )), 
			this, SLOT(openChatWin( const QString& )));
	} else {
		w_buddyList->raise();
	}

	list = g_cxConxion->getOnlineUsers();
	w_buddyList->slotPopulateOnline( list );
	list.clear();

	list = g_cxConxion->getAllUsers();
	w_buddyList->slotPopulateUsers( list );
	list.clear();

	w_buddyList->show();
}

/**
 ** CxDaemonTray::openChatWin(): Start chatting with a user.
 **/
void		CxDaemonTray::openChatWin( const QString& userName ) {
CxWndChat	*win;

	win = &chatWin( userName );
	win->show();
}

/**
 ** CxDaemonTray::closeChatWin(): Start chatting with a user.
 **/
void		CxDaemonTray::closeChatWin( const QString& userName ) {

	kdDebug() << "Closing chatwin for " << userName << endl;
	if( w_chatWindows[userName] ) {
		delete w_chatWindows[userName];
		w_chatWindows.remove(userName);
	} else {
		kdDebug() << "No chatwin open for " << userName << endl;
	}
}

/**
 ** CxDaemonTray::chatWin(): Return a pointer to a fresh new chat window, or to an
 ** existing window.
 **/
CxWndChat	&CxDaemonTray::chatWin( const QString& userName ) {
CxWndChat	*nu;

	kdDebug() << "locate chat window for username " << userName << endl;
	nu = w_chatWindows[userName];
	if(!nu) {
		kdDebug() << "CREATE chat window for username " << userName << endl;
		nu = new CxWndChat( userName );
		nu->setUserName( g_cxConxion->getUserName() );
		connect(nu, SIGNAL(closing(const QString &)), this, SLOT(closeChatWin(const QString &)) );
		connect(nu, SIGNAL(sendMessage(const QString &, const QString &)), g_cxConxion, SLOT(sendExpressMsg(const QString &, const QString &)) );
		connect(g_cxConxion, SIGNAL(expMesgFailed()), nu, SLOT(userIsOffline()));

		w_chatWindows.insert(userName, nu);
	} else {
		kdDebug() << "chat window located." << endl;
	}
	return(*nu);
}

/**
 ** CxDaemonTray::slotExpressMsg(): Handle an incoming express message.
 **/
void		CxDaemonTray::slotExpressMsg( const QString& username, const QString& message) {

	kdDebug() << "[Express Message]" << endl;
	kdDebug() << "From: " << username << endl;
	kdDebug() << message << endl << "---" << endl;

	CxWndChat *win = &chatWin( username );
	if( win ) {
		kdDebug() << "-> win->recvMesg()" << endl;
		win->recvMesg( message.ascii() );
		kdDebug() << "-> win->show()" << endl;
		win->show();
	} else {
		kdDebug() << "NO WIN." << endl;
	}
}

/**
 ** CxDaemonTray::raisePrefsDlg(): Open up the Infusion Daemon preferences.
 **/
void		CxDaemonTray::raisePrefsDlg() {

	CxWndPrefs *wndPrefs = new CxWndPrefs( this, 0, this );
	wndPrefs->show();
}

/**
 ** CxDaemonTray::raiseAboutDlg(): Open up an About dialog for this program.
 **/
void		CxDaemonTray::raiseAboutDlg() {

	CxWndAbout *wndAbout = new CxWndAbout( kapp, 0 );
	wndAbout->show();
}

/**
 ** CxDaemonTray::slotGoOffline(): Disconnect from server.
 **/
void		CxDaemonTray::slotGoOffline() {

	emit g_cxConxion->disconnect();
}

/**
 ** CxDaemonTray::slotGoOnline(): Connect to the configured server.
 **/
void		CxDaemonTray::slotGoOnline() {

	emit g_cxConxion->connect();
}

/**
 ** CxDaemonTray::getFloorCache(): Return a reference to the current
 ** floor cache.
 **/
CXLIST		*CxDaemonTray::getFloorCache() {

	return( (g_cxConxion->floorCache()) );
}

/**
 ** CxDaemonTray::setIcon(): Set the active icon.
 **/
void		CxDaemonTray::setIcon( CurrentState state ) {

	/**
	 ** Depending on which state the daemon is in...
	 **/
	switch( state ) {

		case( CxDaemonTray::Online ):
			setPixmap( pix1 );
			break;

		case( CxDaemonTray::Offline ):
			setPixmap( pix2 );
			break;

		case( CxDaemonTray::Working ):
			setPixmap( pix3 );
			break;

		case( CxDaemonTray::NewMail ):
			KNotifyClient::event("NewEmailMsg");
			setPixmap( pixNewMail );
			g_newMailWaiting = true;
			break;

		default:
			setPixmap( pix2 );
			break;
	}
}

/**
 ** CxDaemonTray::setIconOnline(): Make the online icon active.
 **/
void		CxDaemonTray::setIconOnline() {

	emit setIcon( CxDaemonTray::Online );
	emit addToolTip( i18n("Online - ")+g_cxConxion->getHostName() );
	contextMenu()->setItemEnabled( refreshId, TRUE );
//	emit autoSyncLists();
}

/**
 ** CxDaemonTray::setIconNormal(): Make the online icon active, without throttling CxDaemon.
 **/
void		CxDaemonTray::setIconNormal() {

	if(!g_isWorking) return;
	g_isWorking = false;
	emit addToolTip( i18n("Online - ")+g_cxConxion->getHostName() );
	emit setIcon( CxDaemonTray::Online );
	contextMenu()->setItemEnabled( refreshId, TRUE );

	if(!g_firstRun) {
		g_firstRun = true;
		emit checkMail();
	}
}

/**
 ** CxDaemonTray::setIconWorking(): Make the online icon active.
 **/
void		CxDaemonTray::setIconWorking() {

	if(g_isWorking) return;
	g_isWorking = true;
	emit setIcon( CxDaemonTray::Working );
	emit addToolTip( i18n("Working... - ")+g_cxConxion->getHostName() );
	contextMenu()->setItemEnabled( refreshId, FALSE );
}

/**
 ** CxDaemonTray::setIconOffline(): Make the offline icon active.
 **/
void		CxDaemonTray::setIconOffline() {

	emit setIcon( CxDaemonTray::Offline );
	emit addToolTip( i18n("Offline") );
	contextMenu()->setItemEnabled( refreshId, FALSE );
}

/**
 ** CxDaemonTray::setIconNewMail(): The user has new e-mail.
 **/
void		CxDaemonTray::setIconNewMail() {

	if( !g_newMailWaiting ) {
		KNotifyClient::event(QString("New E-mail Message"));
	}
	emit setIcon( CxDaemonTray::NewMail );
	emit addToolTip( i18n("You have new mail!") );
}

/**
 ** CxDaemonTray::checkMail(): Check for new e-mail messages on all connections.
 **/
void		CxDaemonTray::doCheckMail() {

	emit g_cxConxion->checkMail();

	/**
	 ** Note that QTimer takes seconds in utime (that's 'microseconds'
	 ** to you, Brian).  Remember this when setting the preferences
	 ** from the user.  u_long gp_mailCheckInterval ?
	 **/
	QTimer::singleShot(300000, this, SIGNAL( checkMail() ));
}

/**
 ** CxDaemonTray::configurationChanged(): We will need to drop our connection
 ** to the backend hosts and reconnect when this happens.
 **/
void		CxDaemonTray::configurationChanged() {
KConfig		*kc = kapp->config();
QString		tmp;

	kc->setGroup("Accounts");
	tmp = kc->readEntry( "Configured" );
	if(!tmp.isNull()) {
		tmp = kc->readEntry( "Host" );
		if(!tmp.isNull()) g_cxConxion->setHostName( tmp );
		tmp = kc->readEntry( "Username" );
		if(!tmp.isNull()) g_cxConxion->setUserName( tmp );
		tmp = kc->readEntry( "Save Password" );
		if(!tmp.isNull()) {
			tmp = kc->readEntry( "Password" );
			if(!tmp.isNull()) g_cxConxion->setPassword( tmp );
		} else {
			g_cxConxion->setPassword( QString::null );
		}
	}

	emit goOffline();
	emit goOnline();
}

/**
 ** CxDaemonTray::getUserName(): Obtain the username for connection [x].  This will
 ** need to be modified in order to accomplish connection parallelization. [PARASPT]
 **/
QString		CxDaemonTray::getUserName() {

	return( g_cxConxion->getUserName() );
}

/**
 ** CxDaemonTray::fetchNotes(): Retrieve the notes from the server. [PARASPT]
 **/
QCStringList	CxDaemonTray::fetchNotes() {
QCStringList	res;

	res = g_cxConxion->fetchNotes();
	return( res );
}

/**
 ** CxDaemonTray::fetchMessageList(): Retrieve a list of messages for the specified
 ** room.  This is an empty frontend for the Connection layer.  [PARASPT]
 **/
QCStringList	CxDaemonTray::fetchMessageList( const QString &room, bool cache_results ) {
QCStringList	res;

	res = g_cxConxion->fetchMessageList( room, cache_results );
	return( res );
}

/**
 ** CxDaemonTray::fetchFolderList(): Retrieve a list of folders on the specified
 ** host.  [PARASPT]
 **/
QCStringList	CxDaemonTray::fetchFolderList() {
QCStringList	res;

	res = g_cxConxion->fetchFolderList();

	return( res );
}

/**
 ** CxDaemonTray::deliverMessage(): Deliver this message according to our delivery
 ** rules [PARASPT]
 **/
int		CxDaemonTray::deliverMessage( QCStringList *msg ) {

	return( g_cxConxion->deliverMessage( msg ) );
}

/**
 ** CxDaemonTray::fetchMessage(): Retrieve the specified message.  [PARASPT]
 **/
QCStringList	CxDaemonTray::fetchMessage( const QCString &room, long unsigned int msg_id ) {
QCStringList	res;

	emit setIconWorking();
	res = g_cxConxion->fetchMessage( room, msg_id );
	emit setIconNormal();
	return(res);
}

/**
 ** CxDaemonTray::cacheStats(): Display statistics for the cache subsystem.  [PARASPT]
 **/
QCStringList	CxDaemonTray::cacheStats() {
QCStringList	res;

	kdDebug() << "CxDaemonTray::cacheStats(): Called" << endl;
	res = g_cxConxion->cacheStats();
	return( res );
}
