/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: wndchat.o
 ** Date: 2000-10-15
 ** Last Revision: 2000-10-15
 ** Description: Express Message chat window.
 ** CVS: $Id: wndchat.cpp,v 1.2 2001/07/25 02:22:17 brian Exp $
 **/
#define         __CX_SOURCEFILE__
#include        <unistd.h>
#include        <stdlib.h>
#include        <stdio.h>
#include        <string.h>
#include        <iostream.h>
#include        <qpushbutton.h>
#include        <qsplitter.h>
#include        <kglobal.h>
#include        <klineedit.h>
#include        <kaccel.h>
#include        <kdialog.h>
#include        <ktabctl.h>
#include        <kmessagebox.h>
#include        <qstring.h>
#include        <CxClient.h>
#include	"wndchat.h"
#define		ID_SEND		0x1

/**
 ** CxWndChat::CxWndChat(): Constructor
 **/
		CxWndChat::CxWndChat(const QString& to) :
			KMainWindow(0, 0) {

	KAccel *keys = new KAccel(this);
	keys->connectItem(KStdAccel::Close, this, SLOT(close()));
	keys->connectItem(KStdAccel::Quit, this, SLOT(close()));
	keys->readSettings();

	thisUser = QString::null;
	remoteUser = to;
	setCaption(QString("Chatting with ")+to);

	file = new QPopupMenu();
	file->insertItem("&Send Message", this, SLOT(onSend()));
	file->insertSeparator();
	file->insertItem("&Close", this, SLOT(close()));  
	menuBar()->insertItem("&File", file);

	page = new QVBox(this);
	page->setMargin(KDialog::marginHint());
	page->setSpacing(KDialog::spacingHint());
	setCentralWidget(page);

	txtBody = new QTextBrowser(page,"wndChatList");

	QHBox *hb = new QHBox(page);
	lString = new KLineEdit(hb,"wndChatEntry");
	lString->setHandleSignals( false );
	lString->setCompletionMode( KGlobalSettings::CompletionNone );
	lString->setFocus();
	QPushButton *btn = new QPushButton(QString("Send"), hb, "wndChatSend");
	connect(btn, SIGNAL(clicked()), this, SLOT(onSend()));
	connect(lString, SIGNAL(returnPressed()), this, SLOT(onSend()) );

	resize(250, 200);
}

/**
 ** CxWndChat::appendMessage(): Append a message to the Chat Buffer.
 **/
void		CxWndChat::appendMessage(const QString& msg) {

	txtBody->append( msg );
	txtBody->ensureVisible(0, txtBody->contentsHeight());
}

/**
 ** CxWndChat::clearEntry(): Clear the text entry box.
 **/
void		CxWndChat::clearEntry() {

	lString->clear();
	lString->setFocus();
}

/**
 ** CxWndChat::recvMsg(): Receive message
 **/
void		CxWndChat::recvMesg(const char *s) {

	emit appendMessage(QString("<FONT COLOR=\"#AA0000\"><B>") + remoteUser + QString(":</B></FONT>  ") + QString(s) + QString("<BR>\n"));
}

/**
 ** CxWndChat::onSend(): Send message
 **/
void		CxWndChat::onSend() {

	// Don't waste my time sending blank messages.
	if(lString->text().isEmpty()) {
		return;
	}

	emit appendMessage(QString("<FONT COLOR=\"#0000AA\"><B>") + thisUser + QString(":</B></FONT>  ") + lString->text() + QString("<BR>\n"));
	emit sendMessage( remoteUser, lString->text() );
	emit clearEntry();
}

/**
 ** CxWndChat::userIsOffline(): Inform the user that the remote user is offline.
 **/
void		CxWndChat::userIsOffline() {

	emit appendMessage(QString("<HR><I>User ")+remoteUser+QString(" is not online.</I><P>\n"));
	lString->setDisabled(true);
}

/**
 ** CxWndMain::closeEvent(): What do we do when we exit?
 **/
void            CxWndChat::closeEvent(QCloseEvent *) {

	emit closing(remoteUser);
}
