#include	<iostream.h>
#include	<qbuttongroup.h>
#include	<qgroupbox.h>
#include	<qlineedit.h>
#include	<qcombobox.h>
#include	<qlayout.h>
#include	<qlabel.h>
#include	<qheader.h>
#include	<qcheckbox.h>
#include	<qapp.h>
#include	<qvbox.h>
#include	<qwidget.h>
#include	<kdialogbase.h>
#include	<klocale.h>
#include	<ksimpleconfig.h>
#include	<kconfig.h>
#include	<kglobal.h>
#include	<kinstance.h>
#include	<kiconloader.h>
#include	<klistview.h>
#include	<kglobalsettings.h>
#include	<ktabctl.h>
#include	"cxdaemontray.h"
#include	"wndprefs.h"

/**
 ** CxWndPrefs::CxWndPrefs(): CONSTRUCTOR. Preferences dialog.  Hints taken
 ** from KMail.
 **/
		CxWndPrefs::CxWndPrefs(QWidget *parent, const char *name, CxDaemonTray *trayParent)
		:KDialogBase( IconList, i18n("Preferences"), Help|Default|Apply|Ok|Cancel, Ok, parent,
				name, true, true ) {

	mpTray = trayParent;
	connect(this, SIGNAL(configChanged()), mpTray, SLOT(configurationChanged()));

	setHelp( "infusion/configure.html", QString::null );
	setIconListAllVisible( true );
	enableButton( Default, false );

	connect(this, SIGNAL(applyClicked()), this, SLOT(applyChanges()));
	connect(this, SIGNAL(okClicked()), this, SLOT(saveAndExit()));

	createTabIdentity();
	createTabNotification();

	loadSettings( kapp->config() );

	int W=QApplication::desktop()->width();
	int H=QApplication::desktop()->height();
	int w=width();
	int h=height();

	move( ((W/2)-(w/2)), ((H/2)-(h/2)) );
}

/**
 ** CxWndPrefs::createTabIdentity(): Create the "Identity" pane.
 **/
void		CxWndPrefs::createTabIdentity() {

	QFrame *page = addPage( i18n("Identity"), i18n("Personal Information"),
		KGlobal::instance()->iconLoader()->loadIcon("identity", KIcon::NoGroup, KIcon::SizeMedium ));
	QVBoxLayout *topLevel = new QVBoxLayout( page, 0, spacingHint() );
	QGridLayout *layout = new QGridLayout( topLevel, 12, 3 );
	layout->addColSpacing(1, fontMetrics().maxWidth()*15 );
	layout->addRowSpacing(6, spacingHint() );
	layout->setRowStretch( 11, 10 );
	layout->setColStretch( 1, 10 );

	QLabel *label = new QLabel( i18n("Identity:"), page );
	layout->addWidget( label, 0, 0 );

	QVBox *hb = new QVBox( page );
	layout->addWidget( hb, 0, 1 );

	KListView *klv = new KListView( hb );
	klv->addColumn( QString::null );
	klv->addColumn( QString::null );
	klv->setFixedHeight( 50 );
	klv->header()->hide();
	QListViewItem *i = new QListViewItem( klv, QString("Primary Identity (Default)"), QString("Citadel/UX") );
	i->setPixmap( 0, SmallIcon("kdmconfig") );

	QHBox *vb = new QHBox( hb );
	QPushButton *pb1 = new QPushButton( QString("&Add"), vb );
	pb1->setEnabled( false );
	QWidget *spacer = new QWidget( vb );
	spacer->setFixedWidth( 6 );
	pb1 = new QPushButton( QString("&Remove"), vb );
	pb1->setEnabled( false );
	
	label = new QLabel( i18n("Citadel Host:"), page );
	layout->addWidget( label, 1, 0 );

	t_Host = new QComboBox( true, page );
	t_Host->setInsertionPolicy( QComboBox::AtTop );
	t_Host->insertItem( SmallIcon("host"), i18n("uncensored.citadel.org") );
	t_Host->insertItem( SmallIcon("host"), i18n("pixel.citadel.org") );
	layout->addMultiCellWidget( t_Host, 1, 1, 1, 2 );

	label = new QLabel( i18n("Real Name:"), page );
	layout->addWidget( label, 2, 0 );

	t_fullName = new QLineEdit( page );
	layout->addWidget( t_fullName, 2, 1 );

	label = new QLabel( i18n("Username:"), page );
	layout->addWidget( label, 3, 0 );

	t_Username = new QLineEdit( page );
	layout->addWidget( t_Username, 3, 1 );

/*
	label = new QLabel( i18n("Password:"), page );
	layout->addWidget( label, 4, 0 );
	t_Password = new QLineEdit( page );
	t_Password->setEchoMode( QLineEdit::Password );
	layout->addWidget( t_Password, 4, 1 );
 */

	t_Password = new QString();

	t_SavePw= new QCheckBox( i18n("&Remember Password"), page );
	layout->addWidget( t_SavePw, 4, 1 );
}

/**
 ** CxWndPrefs::createTabNotification(): Create the "Notification" pane.
 **/
void		CxWndPrefs::createTabNotification() {

	QFrame *page = addPage( i18n("Notification"), i18n("Notification Settings"),
		KGlobal::instance()->iconLoader()->loadIcon("bell", KIcon::NoGroup, KIcon::SizeMedium ));
	QVBoxLayout *topLevel = new QVBoxLayout( page, 0, spacingHint() );
	QGridLayout *layout = new QGridLayout( topLevel, 12, 3 );
	layout->addColSpacing(1, fontMetrics().maxWidth()*15 );
	layout->addRowSpacing(6, spacingHint() );
	layout->setRowStretch( 11, 10 );
	layout->setColStretch( 1, 10 );

	QLabel *label = new QLabel( i18n("Nothing:"), page );
	layout->addWidget( label, 0, 0 );
}

/**
 ** CxWndPrefs::loadSettings(): Load the current settings from disk.
 **/
void		CxWndPrefs::loadSettings( KConfig *mpConfig ) {
QString		tmp;

	mpConfig->setGroup("Accounts");
	tmp = mpConfig->readEntry("Host");
	if(tmp) {
		t_Host->insertItem( SmallIcon("host"), tmp );
		t_Host->setCurrentItem( -1 );
	}

	tmp = mpConfig->readEntry("Username");
	if(!tmp.isNull()) t_Username->setText(tmp);

	tmp = mpConfig->readEntry("Password");
//	if(!tmp.isNull()) t_Password = tmp;

	tmp = mpConfig->readEntry("Save Password");
	if(tmp == "1") t_SavePw->setChecked( true );

	tmp = mpConfig->readEntry("FullName");
	if(!tmp.isNull()) t_fullName->setText(tmp);
}

/**
 ** CxWndPrefs::saveSettings(): Save the current settings.
 **/
void		CxWndPrefs::saveSettings( KConfig *mpConfig ) {
QString		tmp;
bool		config_changed = false, refresh_conn = false;

	mpConfig->setGroup("Accounts");
	mpConfig->writeEntry("Configured", i18n("Yes"));

	tmp = mpConfig->readEntry("Host");
	if( tmp != t_Host->currentText() ) {
		config_changed = refresh_conn = true;
		mpConfig->writeEntry("Host", t_Host->currentText() );
	}

	tmp = mpConfig->readEntry("FullName");
	if( tmp != t_fullName->text() ) {
		config_changed = true;
		mpConfig->writeEntry("FullName", t_fullName->text() );
	}

	tmp = mpConfig->readEntry("Username");
	if( tmp != t_Username->text() ) {
		config_changed = refresh_conn = true;
		mpConfig->writeEntry("Username", t_Username->text() );
	}

	tmp = mpConfig->readEntry("Save Password");
	if(!tmp) tmp = QString("0");

	if( (tmp == "1") != t_SavePw->isChecked() ) {
		config_changed = true;
		mpConfig->writeEntry("Save Password", (t_SavePw->isChecked()?QString("1"):QString("0")) );
	}

	if( config_changed ) mpConfig->sync();
	if( refresh_conn ) emit configChanged();
}

/**
 ** CxWndPrefs::applyChanges(): Apply our changes, don't exit.
 **/
void		CxWndPrefs::applyChanges() {

	emit saveSettings( kapp->config() );
}

/**
 ** CxWndPrefs::saveAndExit(): Save settings and exit.
 **/
void		CxWndPrefs::saveAndExit() {

	hide();
	emit saveSettings( kapp->config() );
	emit close();
}
