/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: modules/ui/kde2/objfolderlist.o
 ** Date: 2000-12-11
 ** Last Revision: 2000-12-11
 ** Description: Folder list component.
 ** CVS: $Id: objfolderlist.cpp,v 1.17 2001/09/08 02:13:49 brian Exp $
 **/
#define         __CX_SOURCEFILE__
extern "C" {
#include	<stdlib.h>
};
#include        <iostream.h>
#include        <qlayout.h>
#include        <qevent.h>
#include        <qhbox.h>
#include        <kapp.h>
#include        <kglobal.h>
#include        <kmessagebox.h>
#include        <kiconloader.h>
#include        <klistview.h>
#include        <dcopclient.h>
#include        <CxClient.h>
#include	"objfolderlist.h"
#include	"wndmain.h"
extern CxWndMain *wndMain;
extern KApplication *g_InfusionApp;

#include	"icons/folder_16x16.xpm"
#include	"icons/drives_16x16.xpm"

#define		QR_PRIVATE		4

/**
 ** CxObjFolderList::CxObjFolderList(): Constructor.
 **/
		CxObjFolderList::CxObjFolderList(QWidget *parent, int mode, const QString& searchSpec, const char *name): 
		KListView(parent, name) {

	if(mode);
	if(searchSpec);

	connect(this, SIGNAL(executed( QListViewItem* )), SLOT(slotSelected( QListViewItem* )));
	setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
	addColumn( QString("Folder List") );
	setAllColumnsShowFocus( true );
	setRootIsDecorated( true );
	setSorting( -1 );
	setAcceptDrops( false );
	setDragEnabled( false );
	setMinimumWidth( 0 );

	_RoomIndex = _FloorIndex = 0;
}

/**
 ** CxObjFolderList::refresh(): Refresh the user listing.  This will behave
 ** differently depending on which mode the selector is in.
 **/
void		CxObjFolderList::refresh() {
QListViewItem   *i, *ii, *iii, *publicRooms, *privateRooms;
QCString        replyType;
QByteArray      replyData;
 
 
	/**
	 ** Clear the list and start over.
	 **/
	clear();

	/**
	 ** Personal Folders is a navigation assistant to the user's
	 ** most frequently used folders, ala Outlook.  For the moment,
	 ** it's entirely static.
	 **/
	i = new QListViewItem(this, QString("Personal Folders"));
	i->setPixmap(0, QPixmap( drives_16x16_xpm ));
	i->setOpen(true);
 
	ii = new QListViewItem(i, QString("Calendar"));
	ii->setPixmap(0, SmallIcon(QString("karm")));
 
	ii = new QListViewItem(i, QString("Contacts"));
	ii->setPixmap(0, SmallIcon(QString("ktalkd")));
 
	ii = new QListViewItem(i, QString("Deleted Items"));
	ii->setPixmap(0, SmallIcon(QString("trashcan_empty")));
 
	ii = new QListViewItem(i, QString("Inbox"));
	ii->setPixmap(0, SmallIcon(QString("kword")));
 
	ii = new QListViewItem(i, QString("Journal"));
	ii->setPixmap(0, SmallIcon(QString("korganizer")));
 
	ii = new QListViewItem(i, QString("Notes"));
	ii->setPixmap(0, SmallIcon(QString("knotes")));
 
	ii = new QListViewItem(i, QString("Outbox"));
	ii->setPixmap(0, SmallIcon(QString("folder_open")));
 
	ii = new QListViewItem(i, QString("Sent Items"));
	ii->setPixmap(0, SmallIcon(QString("folder")));
 
	ii = new QListViewItem(i, QString("Tasks"));
	ii->setPixmap(0, SmallIcon(QString("klipper")));

	/**
	 ** privateRooms: These are the folders which are "hidden" from other
	 ** users.
	 **/
	privateRooms = new QListViewItem(this, QString("Mailbox - ")+wndMain->getUserNameQs());
	privateRooms->setPixmap(0, QPixmap( folder_16x16_xpm ));

	/**
	 ** Mail Inbox is a static link to the user's _MAIL_ folder.
	 **/ 
	ii = new QListViewItem(privateRooms, QString("Mail Inbox"));
	ii->setPixmap(0, SmallIcon(QString("kwrite")));

	/**
	 ** publicRooms: These folders are "public" folders, which can be written
	 ** to and seen by other users on the system.  On an established Citadel/UX 
	 ** server, a vast majority of the folders will be here.
	 **/
	publicRooms = new QListViewItem(this, QString("Public Folders"));
	publicRooms->setPixmap(0, QPixmap( drives_16x16_xpm ));

	/**
	 ** Obtain a current folder index from infusiond.  The daemon will
	 ** refresh it's internal folder index if it feels that this is
	 ** necessary, so we won't worry about checking it.  Of course, if
	 ** we are unable to communicate with infusiond, print out a warning,
	 ** so we can track down what's going on later.
	 **/
	if(g_InfusionApp->dcopClient()->call("infusiond", "CxMessenger", "fetchFolderList()", 0L, replyType, replyData)) {
		QDataStream reply(replyData, IO_ReadOnly);
                if(replyType == "QCStringList") {
			char *ser[20];
			char buf[1024];
			QDict<QString> floorIndex;
			QDict<QListViewItem> publicFloorPtrs, privateFloorPtrs;

			QCStringList result;
			reply >> result;
 
			QCStringList::Iterator resi;
			for( resi = result.begin(); resi != result.end(); ++resi) {
				QListViewItem *item;

				strcpy( buf, (*resi) );
				CxSerialize( buf, (char **)(&ser));

				/**
				 ** If folder DNE, create it.
				 **/
				if(!floorIndex[ser[0]]) {
					floorIndex.insert( ser[0], new QString(ser[1]) );

					item = new QListViewItem( publicRooms, QString( ser[1] ) );
					item->setPixmap(0, QPixmap( folder_16x16_xpm ));
					publicFloorPtrs.insert( ser[0], item );

					item = new QListViewItem( privateRooms, QString( ser[1] ) );
					item->setPixmap(0, QPixmap( folder_16x16_xpm ));
					privateFloorPtrs.insert( ser[0], item );
				}

/*				QString roomName((ser[4]!=0)?
					(QString("<B>")+QString(ser[3])+QString("</B>"))
					:QString( ser[3] ) 
					);*/
				QString roomName( ser[3] );

				iii = new QListViewItem( publicFloorPtrs[ser[0]], roomName );
				iii->setPixmap(0, QPixmap( folder_16x16_xpm ));
			}

			floorIndex.clear();
			publicFloorPtrs.clear();
			privateFloorPtrs.clear();

		} else {
			cerr << "Infusion CxObjFolderList::refresh(): Dang.  Invalid response type." << endl;
		}
	} else {
		cerr << "Infusion CxObjFolderList::refresh(): Dang.  DCOP failed." << endl;
	}
}

/**
 ** CxObjFolderList::resizeEvent(): Return a pointer to the selected row.
 **/
void		CxObjFolderList::resizeEvent(QResizeEvent *qre) {

	KListView::resizeEvent(qre);
	setColumnWidth(0, width()-4);
}

/**
 ** CxObjFolderList::selectedItem(): Return a pointer to the selected row.
 **/
void		CxObjFolderList::slotSelected(QListViewItem *i) {

	if(!i->childCount()) {
		if(i->parent()->text(0) == "Personal Folders") {

			// FIXME: This is not very language-insensitive... Will need to fix later.
			if(i->text(0) == "Inbox") {
				emit wndMain->slotKoolBar(0, 1);

			} else if(i->text(0) == "Calendar") {
				emit wndMain->slotKoolBar(0, 2);

			} else if(i->text(0) == "Contacts") {
				emit wndMain->slotKoolBar(0, 3);

			} else if(i->text(0) == "Tasks") {
				emit wndMain->slotKoolBar(0, 4);

			} else if(i->text(0) == "Notes") {
				emit wndMain->slotKoolBar(0, 6);
			}
		} else {
			emit wndMain->gotoRoom(i->text(0));
		}
	}
}
