/**
 ** infusion - The Desktop Communications Revoution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: objfolderview.o
 ** Date: 2001-02-15
 ** Last Revision: 2001-02-15
 ** Description: Implementation for folder view.
 ** CVS: $Id: objfolderview.cpp,v 1.16 2001/09/08 02:10:54 brian Exp $
 **/
#include	<stdlib.h>
#include	<qwidget.h>
#include	<qdatetime.h>
#include	<klistview.h>
#include	<kapp.h>
#include	<kdebug.h>
#include	<dcopclient.h>
#include	<qsplitter.h>
#include	<klistview.h>
#include	<kiconloader.h>
#include	<klocale.h>
#include	<kmessagebox.h> 
#include	<CxClient.h>
#include	"wndmesg.h"
#include	"wndmain.h"
#include	"objfolderview.h"
#include	"objgenericview.h"
#include	"objmesgpane.h"

#include	"icons/mail_new_16x16.xpm"
#include	"icons/mail_old_16x16.xpm"

extern CxWndMain *wndMain;

CXOBJDEF	CXINBOX = {
	"cxobjfolderview",
	"Inbox",
	"kword"
};

CXOBJDEF	CXTRASH = {
	"cxobjtrashview",
	"Deleted Items",
	"trashcan_empty"
};

/**
 ** CxObjFolderView::CxObjFolderView(): Constructor
 **/
		CxObjFolderView::CxObjFolderView( QWidget *parent, const QString& lockRoom )
		: CxObjGenericView(parent) {

	/**
	 ** A folder can be started up in LOCKED mode, which
	 ** ignores requests to change to a different context.
	 ** This is useful for use with rooms like Trashcan.
	 **/
	if( lockRoom ) lockedInRoom = lockRoom;

	_splitter = new QSplitter( Vertical, this );
	myList = new KListView( _splitter );

        connect(myList, 
		SIGNAL(selectionChanged( QListViewItem* )), 
		SLOT(slotSelected( QListViewItem* )));
        connect(myList, 
		SIGNAL(doubleClicked( QListViewItem* )), 
		SLOT(slotOpenMesg( QListViewItem* )));

        myList->setAllColumnsShowFocus(true);
        myList->setFrameStyle(QFrame::WinPanel + QFrame::Sunken);
        myList->addColumn("MsgID");
        myList->addColumn("");
        myList->addColumn("");
        myList->addColumn("Author");
        myList->addColumn("Date");
        myList->addColumn("Subject");
        myList->setSorting(2);
        myList->setAcceptDrops(false);
        myList->setDragEnabled(false);
        myList->setColumnWidth(0,0);
        myList->setColumnWidth(1,0);
        myList->setColumnWidth(3,16);
        myList->setColumnWidthMode(0, QListView::Manual);
        myList->setColumnWidthMode(1, QListView::Manual);
        myList->setColumnWidthMode(5, QListView::Maximum);
//        myList->setColumnWidthMode(5, QListView::Manual);
	_splitter->setResizeMode(myList, QSplitter::KeepSize);

	myMesgPane = new CxObjMesgPane( _splitter );
	_splitter->setResizeMode(myMesgPane, QSplitter::Stretch);

	if( !lockedInRoom.isNull() ) myMesgPane->hide();

	/**
	 ** (we wish to start out in a NULL room, which will cause our
	 ** gotoRoom() command to be triggered upon initialization.)
	 **/
	if(!lockedInRoom) currentRoom = QString::null;
	else currentRoom = lockedInRoom;

	_MesgIndex = _MesgInfo = _Unread = 0;

	m_contextMenu = new QPopupMenu();
        m_replyId = m_contextMenu->insertItem(SmallIcon("mail_reply"),QString("&Reply..."), 
		this, SLOT(slotShowProperties()));
        m_replyAllId = m_contextMenu->insertItem(SmallIcon("mail_replyall"),QString("&Reply to all..."), 
		this, SLOT(slotShowProperties()));
        m_forwardId = m_contextMenu->insertItem(SmallIcon("mail_forward"),QString("&Forward..."), 
		this, SLOT(slotShowProperties()));
        m_contextMenu->insertSeparator();
        m_propsId = m_contextMenu->insertItem(QString("&Properties..."), 
		this, SLOT(slotShowProperties()));
	connect( 
		myList, SIGNAL(contextMenu (KListView*, QListViewItem*, const QPoint&)),
		this, SLOT(rmbClicked(KListView *,QListViewItem *, const QPoint &)) 
	);

	g_paneIsVisible = true;
}

/**
 ** CxObjFolderView::refresh(): Refresh the message list.
 **/
void		CxObjFolderView::refresh() {
char            buf[255];
LISTVIEWITEM   *i;
QDateTime       td;
QCString	replyType;
QByteArray	data,replyData;

	if(currentRoom.isNull()) currentRoom = QString(wndMain->getRoomName());
	if(currentRoom.isNull() || (currentRoom == "")) {
		wndMain->gotoRoom( "_MAIL_" );
		currentRoom = QString("_MAIL_");
	}
	if( currentRoom == "Mail" ) currentRoom = QString("_MAIL_");

	QDataStream arg(data, IO_WriteOnly);
	arg << (lockedInRoom.isNull()?1:0);
	arg << QString(currentRoom);

	/**
	 ** infusiond maintains the network connection, so we should
	 ** retrieve a list of messages from it.
	 **/
	if(wndMain->dcopClient()->call(
		"infusiond","CxMessenger", 
		"fetchMessageList(int,QString)",
		data,replyType,replyData)
	) {

		QDataStream reply(replyData, IO_ReadOnly);
		if (replyType == "QCStringList" ) {
			QCStringList result;
			char *tmp[20];
			reply >> result;

			QCStringList::Iterator resi;
			myList->clear();
			for( resi = result.begin(); resi != result.end(); ++resi) {

				kdDebug() << "CxObjFolderView::refresh(): resi = " << (*resi) << endl;
				strcpy(buf, (*resi));
				CxSerialize( buf, (char **)&tmp );

				kdDebug() << "CxObjFolderView::refresh(): tmp[3] = " << tmp[3] << endl;
				kdDebug() << "CxObjFolderView::refresh(): tmp[5] = " << tmp[5] << endl;
				if( !QString(tmp[5]).isEmpty() ) { 
					long unsigned int ts = atoi(tmp[5]);
					if(!ts) ts = 0;
					td.setTime_t( ts );
				} else {
					td.setTime_t( 0L );
				}
/*				if(tmp[5][0]) td.setTime_t(atoi(tmp[5]));
				else td.setTime_t( 0 ); */

				bool subj_is_null = false;
				if(!strcmp(tmp[4],"(null)")) subj_is_null = true;
				else if(!tmp[4][0] || !tmp[4]) subj_is_null = true;

				QString author( tmp[2] );

				/**
				 ** Clean up author display...
				 **/
				if( author.find('(') && author.find(')') ) {
					author = author.mid( author.find('(')+1, author.find(')')-author.find('(')-1 );
				} else if( author.find('<') && author.find('>') ) {
					author = author.mid( 0, author.find('<')-1 );
				} else {
					author = tmp[2];
				}

				/**
				 ** ie. 817989|1|IGnatius T Foobar|994991986|(null)
				 **/
				i = new LISTVIEWITEM( myList,
					tmp[0],
					tmp[3],
					QString::null, 
					author, 
					KGlobal::locale()->formatDateTime(td, true, false), // td.toString(),
					((!subj_is_null)?tmp[4]:"(No Subject)")
				);

				/**
				 ** This message is unread.
				 **/
				if(!strcmp(tmp[1], "1")) {
					i->setPixmap(2,QPixmap(mail_new_16x16_xpm));
				} else {
					i->setPixmap(2,QPixmap(mail_old_16x16_xpm));
				}
			}

			result.clear();
		} else {
			cerr << "Infusion CxObjFolderView::refresh(): DCOP response of unknown type." << endl;
		}
	} else {
		cerr << "Infusion CxObjFolderView::refresh(): DCOP call failed!" << endl;
	}
}

/**
 ** CxObjFolderView::slotLoadMessage(): Load a message.  Duh. :)
 **/
void		CxObjFolderView::slotLoadMessage(const QString &room, const QString &messageId) {
MESGINFO	*mi;

	if(messageId);

	if(currentRoom != room) {
		ROOMINFO *ri = CxRmGoto( wndMain->getHndl(), room.ascii(), 0);
		if(ri) {
			currentRoom = QString(ri->name);
			free(ri);
		} else return;
	}

	mi = 0;
}

/**
 ** CxObjFolderView::slotOpenMesg(): What happens when an item is
 ** selected?
 **/
void		CxObjFolderView::slotOpenMesg(QListViewItem *item) {
//MESGINFO        m;

	if(!item) return;
	if(!item->text(0)) return;
	if(!item->text(0).ascii()) return;
	if(!*item->text(0).ascii()) return;

	CxWndMesg *msg = new CxWndMesg( wndMain->getRoomNameQs(), item->text(0).toULong() );
	if(!msg->errorCode()) {
		msg->show();

	} else {
		int err = msg->errorCode();
		delete msg;

		kdDebug() << "CxObjFolderView::CxObjFolderView(): WndMesg returned error code " << err << endl;
		KMessageBox::error(this, 
			i18n("I'm sorry, but I was unable to load the message you requested.\n\nError Code %1").arg(err), 
			i18n("Error Loading Message"),
			false);
	}
}

/**
 ** CxObjFolderView::slotSelected(): What happens when we select an item?
 **/
void            CxObjFolderView::slotSelected(QListViewItem *item) {
QCString	replyType;
QByteArray	replyData;
        
	if(!item) return;
	if(!item->text(0)) return;
	if(!item->text(0).ascii()) return;
	if(!*item->text(0).ascii()) return;

	/**
	 ** Encapsulate the note in a DCOP message, which will
	 ** be gingerly passed to the CxMessegner::deliverMessage()
	 ** call.
	 **/
	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	arg << QCString( wndMain->getRoomNameQs() );
	arg << item->text(0).toULong();

	/**
	 ** Load the requested message from the Infusion Daemon.  If the
	 ** message could not be open, abort bootstrap and display an error
	 ** message.  (Then, close)
	 **/
	if(wndMain->dcopClient()->call("infusiond", "CxMessenger", "fetchMessage(QCString,unsigned int)", data,
					replyType, replyData)) {
 
		QDataStream reply(replyData, IO_ReadOnly);
		if (replyType == "QCStringList" ) {
			//int	elements = 0;
			QCStringList mesgInfo;
			QString	g_From, g_To, g_Subject, g_MessageBody;

			reply >> mesgInfo;

			QCStringList::Iterator x;
			int i = 0;
			for(x = mesgInfo.begin(); x != mesgInfo.end(); ++x) {
				switch(i) {
					case(0): break;
					case(1): break;
					case(2): g_From = (*x); break;
					case(3): g_To = (*x); break;
					case(4): break;
					case(5): g_Subject = (*x); break;
					case(6): g_MessageBody = (*x); break;
					default: kdDebug() << "CxObjFolderView::slotSelected(): excessive data!  my head hurts!" << endl; break;
				}
				++i;
			}

			if( (i == 7) ) {
				emit myMesgPane->slotLoadMesg(
					g_From,
					g_To,
					g_Subject,
					g_MessageBody
				);
			}
		}
	}
}

/**
 ** CxObjFolderView::slotTogglePane(): Toggle the pane.
 **/
void		CxObjFolderView::slotTogglePane() {

	if(g_paneIsVisible) {  //myMesgPane->isVisible()) {
		myMesgPane->hide();
		g_paneIsVisible = false;
	} else {
		myMesgPane->show();
		g_paneIsVisible = true;
	}
}

/**
 ** CxObjFolderView::slotToggleHdr(): Toggle the pane.
 **/
void		CxObjFolderView::slotToggleHdr() {

	myMesgPane->slotToggleHdr();
}

/**
 ** CxObjFolderView::clear(): Clear the FolderView.
 **/
void		CxObjFolderView::clear() {

	emit myList->clear();
	emit myMesgPane->clear();
}

/**
 ** CxObjFolderView::gotoRoom(): Adjust our currentRoom pointer
 ** and refresh our view.
 **/
void		CxObjFolderView::gotoRoom( const QString& room ) {

	if(lockedInRoom.isNull() && (room != currentRoom)) {
		currentRoom = room;
		//emit refresh();
	}
}

/**
 ** CxObjFolderView::slotShowProperties(): Popup a message
 ** properties window for the selected message.
 **/
void		CxObjFolderView::slotShowProperties() {

//	kdDebug() << "CxObjFolderView::slotShowProperties()" << endl;
}

/**
 ** CxObjFolderView::rmbClicked(): Handle context menu request.
 **/
void		CxObjFolderView::rmbClicked( KListView *list, QListViewItem *i, const QPoint& pt ) {

	if(list);

	if(i) {
		m_contextMenu->setItemEnabled( 0, true );
		m_contextMenu->setItemEnabled( 1, true );
		m_contextMenu->setItemEnabled( 2, true );
		m_contextMenu->popup(pt);
	} else {
		m_contextMenu->setItemEnabled( 0, false );
		m_contextMenu->setItemEnabled( 1, false );
		m_contextMenu->setItemEnabled( 2, false );
		m_contextMenu->popup(pt);
	}
}

/**
 ** CxObjFolderView::setSizes(): Set the sizes of our QSplitter.
 **/
void		CxObjFolderView::setSizes( QValueList<int> sizes ) {

	_splitter->setSizes( sizes );
}

/**
 ** CxObjFolderView::sizes(): Get the sizes of our QSplitter.
 **/
QValueList<int>	CxObjFolderView::sizes() {

	return(_splitter->sizes());
}
