/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: modules/ui/kde2/objnote.o
 ** Date: 2000-12-18
 ** Last Revision: 2000-12-18
 ** Description: Notes View component.
 ** CVS: $Id: objnote.cpp,v 1.8 2001/08/28 01:21:35 brian Exp $
 **/
#define         __CX_SOURCEFILE__
extern "C" {
#include	<time.h> 
};
#include        <string.h>
#include        <iostream.h>
#include        <qframe.h>
#include        <qhbox.h>
#include        <qvbox.h>
#include        <qlabel.h>
#include        <qpalette.h>
#include        <qmultilineedit.h>
#include        <kapp.h>
#include        <kglobal.h>
#include        <kdebug.h>
#include        <kmessagebox.h>
#include        <kiconloader.h>
#include        <dcopclient.h>
#include        <CxClient.h>
#include	"objnote.h"
#include	"icons/sizer.xpm"
#include	"icons/closer.xpm"

/**
 ** CxObjNote::CxObjNote(): Note editor.
 ** (Used knotes as a reference.)
 **/
		CxObjNote::CxObjNote(QWidget *parent):
		QMultiLineEdit(parent) {

	setAcceptDrops(true);
	setBackgroundMode(QWidget::PaletteBase);
	setFrameStyle(QFrame::NoFrame);
	setWordWrap(QMultiLineEdit::WidgetWidth);
	setFont(QFont("helvetica",12));
	connect(this, SIGNAL(textChanged()), this, SLOT(slotCatchChanged()));
	isChanged = 0;
}

/**
 ** CxObjNote::slotCatchChanged(): Catch a change event.
 **/
void		CxObjNote::slotCatchChanged() {

	if(text() == "Give me liberty!") 
		setText("Give me Liberty, or GIVE ME DEATH!\n\nThe Welfare state is destroying all common sense.  It's time to end this charade and TAKE BACK WHAT'S OURS!  VOTE REPUBLICAN!  SAVE THE COUNTRY!\0");
		
	isChanged = 1;
}

/**
 ** CxWndNote::CxWndNote(): Create a popup note.
 ** (Used knotes as a reference, simplified)
 **/
		CxWndNote::CxWndNote(QWidget *parent, const QString &text):
		QFrame( parent, "wndNote", WStyle_Customize|WStyle_NoBorderEx|WDestructiveClose ) {

	setMinimumSize(100,100);

	/**
	 ** make this "appear" to be a sticky note.
	 **/
	QPalette stdpalette = palette();
	QColor bgc = Qt::yellow;
	QColor fgc = Qt::black;
	stdpalette.setColor(QColorGroup::Background, bgc);
	stdpalette.setColor(QColorGroup::Base, bgc);
	stdpalette.setColor(QColorGroup::Foreground, fgc);
	stdpalette.setColor(QColorGroup::Text, fgc);
	setPalette(stdpalette);

	vbox = new QVBox(this);
	vbox->setFrameStyle(QFrame::Box);
	vbox->setFrameShadow(QFrame::Plain);
	vbox->setMargin(2);
	vbox->setSpacing(1);

	/**
	 ** Generate "title" bar.  (We have to generate this 'cuz of the fact that we have no frame!)
	 **/
	QHBox *titleBar = new QHBox(vbox);
	titleBar->setSpacing(1);

	lb = new QLabel(titleBar);
	lb->setAlignment(AlignHCenter);
	lb->setText(QString("iNote"));
	lb->installEventFilter(this);
	lb->setMaximumHeight(16);

	nb = new QLabel(titleBar);
	nb->setPixmap(QPixmap(close_xpm));
	nb->setMaximumSize(16,16);
	nb->setAlignment(AlignCenter);
	nb->installEventFilter(this);

	/**
	 ** Note editor!
	 **/
	enote = new CxObjNote(vbox);
	enote->insertAt( text, 0, 0);
	enote->show();

	/**
	 ** Display the timestamp of this note.
	 **/
	QHBox *statusBar = new QHBox(vbox);

	dt = new QDateTime();
	dt->setTime_t( time(NULL) );
	dtl = new QLabel(statusBar);
	dtl->setAlignment(AlignHCenter);
	dtl->setText(dt->toString());
	dtl->setMinimumWidth( 100-16 );
	dtl->show();

	resizer = new QLabel(statusBar);
	resizer->setPixmap(QPixmap(sizer_xpm));
	resizer->setFixedSize(12,12);
	resizer->setMinimumWidth(12);
	resizer->installEventFilter(this);

	/**
	 ** Adjust the "toolbar" colour to be slightly darker than the rest.
	 **/
	setBackgroundColor(bgc.light(120));
	lb->setBackgroundColor(bgc.dark(120));
	nb->setBackgroundColor(bgc.dark(120));

	resize(200,150);
}

/**
 ** CxWndNote::~CxWndNote(): Destructor!
 **/
		CxWndNote::~CxWndNote() {
	delete(nb);
	delete(resizer);
	delete(lb);
	delete(dt);
	delete(dtl);
	delete(enote);
}

/**
 ** CxWndNote::resizeEvent(): What to do if we get resized?
 **/
void		CxWndNote::resizeEvent(QResizeEvent *ev) {

//	QFrame::resizeEvent( ev );
	vbox->resize( ev->size().width(), ev->size().height() );

//	int new_height = height();
//	int new_width  = width();
//	int headerHeight = lb->sizeHint().height();
//	int footerHeight = dtl->sizeHint().height();

//	nb->setGeometry( new_width - headerHeight, 0, headerHeight, headerHeight );
//	lb->setGeometry( 0, 0, new_width - headerHeight, headerHeight );
//	enote->setGeometry( 0, headerHeight, new_width, new_height - headerHeight );
//	dtl->setGeometry( 0, new_height-footerHeight, new_width, new_height );
//	resizer->setGeometry( new_width - footerHeight, new_height-footerHeight, footerHeight, footerHeight );
}

/**
 ** CxWndNote::eventFilter(): Filter out unnecessary garbage.
 **/
bool 		CxWndNote::eventFilter( QObject* o, QEvent* ev) {
QMouseEvent* 	e = (QMouseEvent*)ev;

	if( o == lb ) {
		if( ev->type() == QEvent::MouseButtonRelease ) {
			if( e->button() == LeftButton ) {
		                g_Dragging = false;
		                lb->releaseMouse();
		                raise();
			}
			if (e->button() == MidButton)
			lower();
			return true;
		}

		if( ev->type() == QEvent::MouseButtonPress && e->button() == LeftButton ) {
			m_pointerOffset = e->pos();
			lb->grabMouse(sizeAllCursor);
			return true;
		}

		if( ev->type() == QEvent::MouseMove && lb == mouseGrabber()) {
			if ( g_Dragging ) {
				move( QCursor::pos() - m_pointerOffset );
			} else {
				g_Dragging = (
					(e->pos().x() - m_pointerOffset.x()) *
					(e->pos().x() - m_pointerOffset.x()) +
					(e->pos().y() - m_pointerOffset.y()) *
					(e->pos().y() - m_pointerOffset.y()) >= 9 );
			}
			return true;
		}
		return lb->eventFilter( o, ev );

	// Object being acted upon is resizer.  We need to resize ourself.
	} else if(o == resizer) {

	        if( ev->type() == QEvent::MouseButtonRelease ) {
                        if( e->button() == LeftButton ) {
                                g_Dragging = false;
                                resizer->releaseMouse();
                                raise();
                        }

			if(g_Dragging) {
				this->resize( width() + (e->pos().x() - m_pointerOffset.x()), height() + (e->pos().y() - m_pointerOffset.y()) );
			}

                        if (e->button() == MidButton) lower();
                        return true;
                }

		if( ev->type() == QEvent::MouseButtonPress && e->button() == LeftButton ) {
			m_pointerOffset = e->pos();
			resizer->grabMouse(sizeAllCursor);
			return true;
		}

		if( (ev->type() == QEvent::MouseMove) && (resizer == mouseGrabber())) {
			if ( g_Dragging ) {
				this->resize( width() + (e->pos().x() - m_pointerOffset.x()), height() + (e->pos().y() - m_pointerOffset.y()) );

			} else {
				g_Dragging = (
					(e->pos().x() - m_pointerOffset.x()) *
					(e->pos().x() - m_pointerOffset.x()) +
					(e->pos().y() - m_pointerOffset.y()) *
					(e->pos().y() - m_pointerOffset.y()) >= 9 );
			}
			return true;
		}
		return resizer->eventFilter( o, ev );

	// Close button
	} else if(o == nb) {

		if( (ev->type() == QEvent::MouseButtonRelease) ) {
			emit slotClose();
		}

	}

/*
	if( m_menu && ( ev->type() == QEvent::MouseButtonPress )
            && ( e->button() == RightButton ) ) {
			m_menu->popup( QCursor::pos() );
			return true;
		}
 */

	return QWidget::eventFilter( o, ev );
}

/**
 ** CxWndNote::slotClose(): Close this note.
 **/
void		CxWndNote::slotClose() {
QCString	replyType;
QByteArray	replyData;

	if(enote->hasBeenChanged()) {

		/**
		 ** Encapsulate the note in a DCOP message, which will
		 ** be gingerly passed to the CxMessegner::deliverMessage()
		 ** call.
		 **/
		QByteArray data;
		QDataStream arg(data, IO_WriteOnly);
		QCStringList foo;
		foo.append( QCString("My Notes") );	// msg.room
		foo.append( QCString("") );	// msg.author
		foo.append( QCString("bledbett@yellowbrix.com") );	// msg.rcpt
		foo.append( QCString("") );		// msg.cc
		foo.append( QCString("") );		// msg.subject
		foo.append( QCString(enote->text()) );			// msg.body
		arg << foo;

		if(kapp->dcopClient()->call("infusiond", "CxMessenger", "deliverMessage(QCStringList)", data,
                        replyType, replyData)) {

			QDataStream reply(replyData, IO_ReadOnly);
			if (replyType == "int" ) {
				int x;
				reply >> x;
				if( x ) {
					kdDebug() << "CxWndNote::slotClose(): save failed." << endl;
				} else {
					kdDebug() << "CxWndNote::slotClose(): save successful." << endl;
					close();
				}
			}
		}

	} else {
		close();
	}
}
