/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: modules/ui/kde2/objnotesview.o
 ** Date: 2000-12-18
 ** Last Revision: 2000-12-18
 ** Description: Notes View component.
 ** CVS: $Id: objnotesview.cpp,v 1.10 2001/08/28 01:21:35 brian Exp $
 **/
#include        <stdlib.h>
#include        <iostream.h>
#include        <qlayout.h>
#include        <qhbox.h>
#include        <qsplitter.h>
#include        <kapp.h>
#include        <klocale.h>
#include        <kdebug.h>
#include        <kglobal.h>
#include        <dcopclient.h>
#include        <kmessagebox.h>
#include        <kiconview.h>
#include        <kiconloader.h>
#include        <CxClient.h>
#include	"objnotesview.h"
#include	"objnote.h"
#include	"wndmain.h"

extern CxWndMain *wndMain;

CXOBJDEF	CXNOTES = {
	"cxobjnotesview",
	"Notes",
	"knotes"
};

/**
 ** CxObjNotesView::CxObjNotesView(): Generate a new IconView containing notes.
 **/
		CxObjNotesView::CxObjNotesView(QWidget *parent, const char *name): 
		CxObjGenericView(parent) {

	if(name) ;

	_iconview = new KIconView( this );
	_iconview->setAutoArrange( true );
	connect( _iconview ,SIGNAL(executed(QIconViewItem *)), this, SLOT(slotSelected(QIconViewItem *)));
	connect( _iconview , SIGNAL(rightButtonClicked( QIconViewItem*, const QPoint& )), this, SLOT(rmbClicked(QIconViewItem *, const QPoint &)) );

	m_NoItem = new QPopupMenu();
	m_NoItem->insertItem(SmallIcon(QString("knotes")), QString("New &Note..."), this, SLOT(slotNewNote()));
	m_NoItem->insertSeparator();
	m_NoItem->insertItem(QString("Arrange &Icons..."), NULL );
	m_NoItem->insertItem(QString("&View..."), NULL );

	m_Item = new QPopupMenu();
	m_Item->insertItem(SmallIcon(QString("knotes")), QString("&Open"), this, SLOT(slotSelected( (QIconViewItem *)NULL )));
}

/**
 ** CxObjNotesView::refresh(): Populate the notes list from the server's database.  This process
 ** reads the notes stored in the "My Notes" room, and "_GLOB_NOTES_"
 **/
void		CxObjNotesView::refresh() {
char		*s, *c;
QString		qs, index;
QIconViewItem	*i;
QCString	replyType;
QByteArray	replyData;

	/**
	 ** Obtain a list of messages from infusiond.  Call the
	 ** fetchNotes() function (which queries the server
	 ** without flushing or changing the cached message lists)
	 **/
	if(kapp->dcopClient()->call("infusiond","CxMessenger","fetchNotes()", 0L, replyType, replyData)) {
		_iconview->clear();
		Notes.clear();

		QDataStream reply(replyData, IO_ReadOnly);
		if (replyType == "QCStringList" ) {
			QCStringList result;
			reply >> result;
 
			QCStringList::Iterator resi;
			for( resi = result.begin(); resi != result.end(); ++resi) {

				s = (char *)malloc(strlen(*resi)+1);
				strcpy(s, (*resi));
				qs = QString(*resi);
 
                                c = s;
                                while(*(c)) {
					if(((*(c)=='.') || (*(c)=='?') || (*(c)=='!'))
						&&
						((*(c+1)==' ') || !(*(c+1)))) {
						*(c+1) = 0;
					}
					c++;
				}
 
				index = QString(s);
				Notes.insert(index, new QString(qs));
 
				delete(qs);
 
				i = new QIconViewItem(_iconview, index, DesktopIcon(QString("knotes")));
				free(s);
			}
		} else {
			kdDebug() << "CxObjNotesView::refresh(): <-dcop, unknown response" << endl;
			i = new QIconViewItem(_iconview,"I'm sorry, but I was unable to access your \"My Notes\" folder.  Please contact the system administrator for assistance.", DesktopIcon(QString("knotes")));
			_iconview->insertItem(i);
		}
	} else {
		kdDebug() << "CxObjNotesView::refresh(): <-dcop, fail" << endl;
		i = new QIconViewItem(_iconview,"I'm sorry, but I was unable to access your \"My Notes\" folder.  Please contact the system administrator for assistance.", DesktopIcon(QString("knotes")));
		_iconview->insertItem(i);

	}
}

/**
 ** CxObjNotesView::slotNewNote(): Pop up a blank notes window.
 **/
void		CxObjNotesView::slotNewNote() {

	CxWndNote *newNote = new CxWndNote(NULL, QString::null);
	newNote->show();
}

/**
 ** CxObjNotesView::slotSelected(): Handle a selected (executed) object.
 **/
void		CxObjNotesView::slotSelected(QIconViewItem *qi) {
QString		ci;

	if(!qi) qi = _iconview->currentItem();
	if(Notes[qi->text()]) {
		ci = (Notes[qi->text()])->copy();
		CxWndNote *n = new CxWndNote(NULL, ci);
		n->show();
	} else {
		KMessageBox::error(this, i18n("I'm sorry, but I could not load the note you requested."), i18n("Error Loading Note"),false);
	}
}

/**
 ** CxObjNotesView::slotSelected(): Handle a selected (executed) object.
 **/
void		CxObjNotesView::slotSelected() {
QString		ci;
QIconViewItem	*qi = 0;

	qi = _iconview->currentItem();
	if(Notes[qi->text()]) {
		ci = (Notes[qi->text()])->copy();
		CxWndNote *n = new CxWndNote(NULL, ci);
		n->show();
	} else {
		KMessageBox::error(this, i18n("I'm sorry, but I could not load the note you requested."), 
			i18n("Error Loading Note"),false);
	}
}

/**
 ** CxObjNotesView::rmbClicked(): Context-sensitive Menu
 **/
void		CxObjNotesView::rmbClicked(QIconViewItem *item, const QPoint& pt) {

	if(item) {
//		emit _iconview->selectionChanged(item);
		m_Item->popup(pt);
	} else {
		m_NoItem->popup(pt);
	}
}

/**
 ** CxObjNotesView::viewLarge(): Adjust the icon view properties..
 **
CxObjNotesView::viewLarge() {
}
 **/
