/**
 ** Object for handling RFC2426 objects ("vCards")
 ** Written by B. Ledbetter (brian@shadowcom.net)
 ** Copyright (c) 2000, Flaming Sword Productions
 **
 ** This object class is designed to store vCard information in
 ** memory for various lists, icon boxes, etc.  It is not made
 ** to store data to the filesystem at this point in time.
 **
 ** I have based this class on _my_ interpretation of RFC 2426,
 ** which is visible at www.ietf.org.  There may very well be some
 ** issues with this interpretation, and I apologize for any
 ** problems that this may cause.
 **
 ** begin:vcard
 ** n:Brian Ledbetter
 ** tel;home:lkjasdfasf
 ** email;internet:asdlfkjasd
 ** adr:;;sdlafkj;lasdkj;as;;USA
 ** end:vcard
 **/
#include	<iostream.h>
#include	<qstring.h>
#include	"rfc2426.h"

/**
 ** Rfc2426::Rfc2426(): CONSTRUCTOR.  This function will, using a hack
 ** of a Finite Deterministic State Machine, parse the object data &
 ** seperate all of the pertinent information out into our private
 ** members.
 **/
		Rfc2426::Rfc2426(const QString& obj_data) {

	o_data = new QString(obj_data);
	parser();
}

/**
 ** Rfc2426::parser(): vCard Parser
 **/
void		Rfc2426::parser() {
QString		tmp;
uint		start_pos, end_pos, smstate;

	smstate = 0;
	start_pos = end_pos = 1;

	cerr << "Rfc2426() - Copyright (c) 2000, Flaming Sword Productions" << endl;
	tmp = o_data->copy();

	cerr << o_data->length() << endl;
	while((start_pos < o_data->length()) && (end_pos < o_data->length())) {
		cerr << o_data->length() << " (" << start_pos << "," << end_pos << ") [" << o_data->mid(start_pos, end_pos-start_pos) << "]";
		/**
		 ** Determine machine state.
		 **/
		switch(smstate) {

			// State: Reading LABEL.
			case(0):
//			switch(o_data[end_pos]) {
//
//				// End of LABEL.
//				case(':'):
//					cerr << "Colon";
//					smstate = 1;
//					break;
//				case('\\'):
//					smstate = 0xff;
//					break;
//			}
			break;

			// State: Read LABEL, Interpret it.
			case(1):
				cerr << "LABEL: " << o_data->mid(start_pos, end_pos-start_pos);
				start_pos = end_pos;
				smstate = 2;
				break;

			case(2):
				smstate = 0;
				break;

			// Illegal state.  Nuke myself.
			default:
				delete(this);
				break;

		}
//		cerr << endl;
		end_pos++;
	}
}
