/**
 ** Object for handling RFC2426 objects ("vCards")
 ** Written by B. Ledbetter (brian@shadowcom.net)
 ** Copyright (c) 2000, Flaming Sword Productions
 **
 ** This object class is designed to store vCard information in
 ** memory for various lists, icon boxes, etc.  It is not made
 ** to store data to the filesystem at this point in time.
 **
 ** I have based this class on _my_ interpretation of RFC 2426,
 ** which is visible at www.ietf.org.  There may very well be some
 ** issues with this interpretation, and I apologize for any
 ** problems that this may cause.
 **/
#ifndef		__RFC2426_H__
#define		__RFC2426_H__
#include	<qstring.h>

class		Rfc2426 {

public:
			// Create object from text stream.
			Rfc2426(const QString& card_data);

private:

	void		parser();


	QString		*o_data,	// Raw, unmolested object data.

			/**
			 ** These data elements are part of the
			 ** VCARD RFC.
			 **/
			*fn,		// Full Name	*REQ*
			*n,		// Name		*REQ*
			*nickname,	// Nickname
			*adr,		// Address
			*label,		// Label/Title ???
			*tel,		// Telephone Number
			*email,		// E-mail Address
			*mailer,	// ???
			*tz,		// Timezone
			*geo,		// Geographic Location ???
			*title,		// Title
			*role,		// Role
			*org,		// Organization
			*categ,		// Categories
			*note,		// Personal Note
			*prodid,	// ???
			*rev,		// ???
			*sort_string,	// ???
			*url,		// User homepage
			*uid,		// User ID ???
			*version,	// ???		*REQ*
			*objclass;	// ???

};

#endif
