/**
 ** Object for handling RFC2426T objects ("vTasks")
 ** Written by B. Ledbetter (brian@shadowcom.net)
 ** Copyright (c) 2000, Flaming Sword Productions
 **
 ** $Id: rfc2426t.cpp,v 1.5 2001/09/08 02:05:56 brian Exp $
 **
 ** This object is designed to store RFC2426T objects, which is MY
 ** variation on the original RFC2426 (vCal) specifications.  This is
 ** NOT intended to be an industry standard, it is simply a dirty kludge
 ** to give me something to work with.
 **
 ** My specs look something like this:
 **
 **     begin:vtask;
 **     n:Go to the grocery store;
 **     dt:2000-01-01 08:11:13;
 **     dtd:2000-01-01 08:11:13;
 **     det:I've got to go to the grocery store to pick up stuff.;
 **     pct:0;
 **     end:vcard;
 **/
extern "C" {
#include	<stdlib.h>
#include	<string.h>
};
#include	<iostream.h>
#include	<qstring.h>
#include	<qdict.h>
#include	"rfc2426t.h"

/**
 ** Rfc2426t::Rfc2426t(): CONSTRUCTOR.  This function will, using a hack
 ** of a Finite Deterministic State Machine, parse the object data &
 ** seperate all of the pertinent information out into our private
 ** members.
 **/
		Rfc2426t::Rfc2426t(const QString& obj_data, const QString& subj, const QString& details) {

	if(subj);
	if(details);

	if(!obj_data.isEmpty()) {
		o_data = new QString(obj_data);
		parser();
//		cout << "[vTask Object]" << endl;
//		cout << "d_Name: " << d_Name << endl;
//		cout << "d_Details: " << d_Details << endl;
	} else {
		o_data = new QString();
//		cout << "empty [vTask Object] Created" << endl;
	}
}

/**
 ** Rfc2426t::~Rfc2426t(): DESTRUCTOR.
 **/
		Rfc2426t::~Rfc2426t() {

	delete o_data;
}

/**
 ** Rfc2426t::parser(): vTask Parser
 **/
void		Rfc2426t::parser() {
uint		start_pos, end_pos, smstate;
const char	*tmpstr;
QString		Label, Value;
QDict<QString>	Variables;

	// Abort if no input vTask object.
	if(o_data->isNull()) return;

	smstate = 0;
	start_pos = 0;
	end_pos = 0;

	cerr << "Rfc2426t() Parser - Copyright (c) 2000, Flaming Sword Productions" << endl;
	tmpstr = o_data->ascii();

//	cout << "Entering state machine." << endl;
//	cout << "PARSING: " << tmpstr << endl << endl;

	while((start_pos < strlen(tmpstr)) && (end_pos < strlen(tmpstr))) {

		/**
		 ** Determine machine state.
		 **/
		//cout << "St[" << smstate << "] Character (" << start_pos << ", "<< end_pos << ") " << tmpstr[end_pos] << endl;
		switch(smstate) {

			/**
			 ** State 0: Reading LABEL.
			 **/
			case(0):
			switch(tmpstr[end_pos]) {

				/**
				 ** End of LABEL.
				 **/
				case(':'):
					Label = o_data->mid(start_pos, end_pos-start_pos).copy();
					start_pos = end_pos;
					smstate = 1;
					break;

				/**
				 ** Newline.  Skip dead space.
				 **/
				case('\n'):
					start_pos = end_pos+1;
					break;

				/**
				 ** ILLEGAL CHARACTER '\'
				 **/
				case('\\'):
					smstate = 0xff;
					break;
			}
			break;

			/**
			 ** State 1: Obtain the VALUE.
			 **/
			case(1):
				switch(tmpstr[end_pos]) {
					case(';'):
						end_pos--;
						smstate = 2;
						break;

					case('\\'):
						smstate = 3;
						break;
				}
				break;

			/**
			 ** State 2: Annotate value.
			 **/
			case(2):
				Value = QString(o_data->mid(start_pos+1, (end_pos-start_pos)-1));
				//cout << "Var: " << Label << " Value:" << Value << endl;
				Variables.insert(Label, new QString(Value));
				end_pos++;
				start_pos = end_pos;
				while(tmpstr[start_pos] == '\n') { start_pos++; end_pos++; };
				smstate = 0;
				break;

			/**
			 ** State 3: In value statement, Encountered '\'
			 **/
			case(3):
				break;

			/**
			 ** *** Illegal state.  Nuke myself, cuz I'm no good.
			 **/
			default:
//				cout << "Illegal state encountered.  Jettison object." << endl;
				delete(this);
				break;
		}
		end_pos++;
	}
//	cout << "State machine exited." << endl;

	/**
	 ** Ensure that we have a valid vTask object.
	 **/
	if(Variables[QString("begin")] && Variables[QString("end")]) {
//		cout << "Valid vTask object?" << endl;

		if(
			(Variables[QString("begin")]->compare(QString("vtask"))) && 
			((Variables[QString("end")])->compare(QString("vtask")))) {

//			cout << "Valid vTask object." << endl;

			d_Name = (Variables[QString("n")])->copy();

			if(Variables[QString("details")]) {
				d_Details = (Variables[QString("details")])->copy();
			}

			if(Variables[QString("pct")]) {
				d_Pct = atof((Variables[QString("pct")])->ascii());
			}
		}
	}
	Variables.clear();
}

/**
 ** Rfc2426t::export_ascii(): Export the object data in ASCII format.  Doesn't do ANYTHING
 ** fancy at this point in time.  :)
 **/
QString		Rfc2426t::export_ascii() {
QString		toRet;

	// n: is a required field.
	if(d_Name.isNull()) {
		return(QString::null);
	}

	cerr << "Rfc2426t() ASCII Exporter - Copyright (c) 2000, Flaming Sword Productions" << endl;

	toRet = QString();

	toRet.append(QString("begin:vTask;\n"));
	toRet.append(QString("n:")+d_Name+QString(";\n"));
	if(!d_Details.isNull()) toRet.append(QString("details:")+d_Details+QString(";\n"));
	toRet.append(QString("pct:")+QString(d_Pct)+QString(";\n"));
	toRet.append(QString("end:vTask;\n"));

	return(toRet);
}

