/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: sysmgmt.o
 ** Date: 2001-02-05
 ** Last Revision: 2001-02-05
 ** Description: Management subsystem.
 ** CVS: $Id: sysmgmt.cpp,v 1.3 2001/05/16 22:40:44 brian Exp $
 **
 ** Management Subsystem - This module allows administrators
 ** to configure basic features of the Infusion client.  This
 ** allows Network Administrators to have better control over
 ** all of their clients, without having to configure each
 ** individual machine.  Please refer to the Infusion 
 ** administrators documentation for details on how this
 ** process works.
 **/
extern "C" {
#include	<stdio.h> 
#include	<stdlib.h> 
#include	<time.h> 
#include	<netdb.h> 
};
#include        <string.h>
#include        <iostream.h>
#include        <qframe.h>
#include        <qhbox.h>
#include        <qvbox.h>
#include        <qlabel.h>
#include        <qpalette.h>
#include        <qmultilineedit.h>
#include        <kapp.h>
#include        <kglobal.h>
#include        <kmessagebox.h>
#include        <kiconloader.h>
#include        <CxClient.h>
#include	"sysmgmt.h"

#define		MACHINE		switch(mstate)
#define		STATE(a)	case(a)

/**
 ** _CxMgtCheck(): Check to see if a Management Server is available
 ** on the network.  This checks to see if the Network Administrator
 ** has defined a server called "citux-manager".  On a typical
 ** Citadel/UX installation, this DNS entry should be a CNAME
 ** (Canonical Name) pointer to your master Citadel/UX server.
 **/
static
int		_CxMgtCheck() {

	if(gethostbyname("citux-manager")) return(1);
	else return(0);
}

/**
 ** CxMgtRetrieve(): Retrieve Management settings from the network.
 ** This function will automatically retrieve configuration settings
 ** set by the Citadel/UX Administrator.  The Management server is
 ** a Citadel/UX server (either the primary, or a dedicated host) which
 ** provides Infusion.inf information to its clients.  Because host-based
 ** TCP/IP is assumed to be available on all Infusion platforms, this
 ** system will be looked up via DNS through the local resolver.  If
 ** this causes excessive network latency, or if you do not have a
 ** network connection, you may disable this feature.
 **/
MGMTINFO	*CxMgtRetrieve() {
char		*s, *c, ch,
		directive[255],
		dirvalue[255];
int		mgmt_info = 0,
		mstate = 0,
		hndl = 0;
MGMTINFO	*toret;

	/**
	 ** If we are unable to locate the Citadel/UX Manager host,
	 ** we don't have to worry about anything.
	 **/
	if(!_CxMgtCheck()) return(NULL);

	hndl = CxClConnection( "citux-manager", 504, NULL, NULL );
	if(!hndl) return(NULL);

	/**
	 ** Attempt to connect to the "citux-manager" server,
	 ** wherever it may be defined on the local DNS server.
	 **/
	if(!CxClConnect( hndl )) {

		printf("** Retrieving management information from Network **\n");

		toret = (MGMTINFO *)malloc(sizeof(MGMTINFO));
		bzero(toret, sizeof(MGMTINFO));

		c = (char *)malloc(strlen("Infusion.inf")+1);
		strcpy(c, "Infusion.inf");

		/**
		 ** If we have connected to the management server,
		 ** we should request the Infusion.inf file from
		 ** it.  If, by some chance, this file can't be
		 ** obtained from the server, we can disconnect.
		 **/
		while((s = CxMiMessage(hndl, c))) {

			if(c) {
				free(c);
				c = NULL;
			}

			mgmt_info = 1;

			c = s;
			mstate = 0;
			bzero(&directive, 254);
			bzero(&dirvalue, 254);
			while(*(c)) {
				ch = *(c);
				MACHINE {

					/**
					 ** STATE 0: WHITESPACE0 State
					 **/
					STATE(0):
					switch(ch) {
						case(';'):
							mstate = 6;
							break;

						case(' '):
						case('\t'):
							break;

						default:
							directive[0] = ch;
							mstate = 1;
							break;
					}
					break;

					/**
					 ** STATE 1: DIRECTIVE State
					 **/
					STATE(1):
					switch(ch) {
						case(' '):
							mstate = 2;
							break;

						case('='):
							mstate = 5;
							break;

						default:
							directive[strlen(directive)] = ch;
							break;
					}
					break;

					/**
					 ** STATE 2: WHITESPACE2 State
					 **/
					STATE(2):
					switch(ch) {
						case('='):
							mstate = 5;
							break;
					}
					break;

					/**
					 ** STATE 3: WHITESPACE3 State
					 **/
					STATE(3):
					switch(ch) {
						case(' '):
							mstate = 3;
							break;

						default:
							dirvalue[0] = ch;
							mstate = 5;
							break;
					}
					break;

					/**
					 ** STATE 5: VALUE State
					 **/
					STATE(5):
						if((ch == ';') || (ch == '\r')) {
							mstate = 6;
						} else if((ch == ' ') && !strlen(dirvalue)) {
							mstate = 3;
						} else {
							dirvalue[strlen(dirvalue)] = ch;
						}
						break;

					/**
					 ** STATE 6: COMMENT State
					 **/
					STATE(6):
						/**
					 	 ** There is NO escape
						 ** from STATE 6.  (Until
						 ** the next line, that is)
						 **/

						break;
				}
				c++;
			}

			/**
			 ** Process the output of this machine for the previous
			 ** line of text.
			 **/
			if(directive[0]) {
				printf("Directive: %s\n", directive);
				printf("Value----: %s\n", dirvalue);

				/**
				 ** Let new users sign up for user accounts?
				 **/
				if(!strcmp(directive, "allow_newusers")) {
					if(!strcmp(dirvalue, "yes")) {
						toret->o_NewUser = 1;
					} else {
						toret->o_NewUser = 0;
					}

				/**
				 ** Allow users to specify connection options?
				 **/
				} else if(!strcmp(directive, "allow_options")) {
					if(!strcmp(dirvalue, "yes")) {
						toret->o_Options = 1;
					} else {
						toret->o_Options = 0;
					}

				} else if(!strcmp(directive, "savepw")) {
					if(!strcmp(dirvalue, "yes")) {
						toret->o_SavePw = 1;
					} else if(!strcmp(dirvalue,"no")) {
						toret->o_SavePw = 0;
					} else {
						toret->o_SavePw = 2;
					}

				}
			}

			/**
			 ** Throw away this line.
			 **/
			c = 0;
			free(s);
		}

		printf("** Done! **\n");

		CxClDisconnect( hndl );
		CxClDelete( hndl );
		return(toret);
	}

	CxClDelete( hndl );
	return(NULL);
}
