#ifndef		__WNDMAIN_H__
#define		__WNDMAIN_H__
#include	<iostream.h>
#include	<qpushbutton.h>
#include	<qsplitter.h>
#include	<qwidgetstack.h>
#include	<qtextbrowser.h>
#include	<qpopupmenu.h>
#include	<dcopclient.h>
#include	<kmainwindow.h>
#include	<kmenubar.h>
#include	<kprogress.h>
#include	<khelpmenu.h>
#include	<ktoolbar.h>
#include	<kstatusbar.h>
#include	<klistview.h>
#include	<kcontainer.h>
#include	"objuserlist.h"
#include	"objmesgpane.h"
#include	"objfolderlist.h"

#include	"objaddrbookview.h"
#include	"objfolderview.h"
#include	"objnotesview.h"
#include	"objtasklist.h"
#include	"objtitlebar.h"
#include	"autoconf.h"
#include	<CxClient.h>

#ifdef		HAVE_KOOLBAR
#include	<koKoolBar.h>
#endif

void    	inChatCallback(const char *user, const char *msg);

class CxWndMain : public KMainWindow
{
  Q_OBJECT
public:
  		CxWndMain();
  void 		closeEvent(QCloseEvent *);
  void 		setConnected(int);
  int 		isConnected();
  void		setStatusMsg(const QString&);
  inline const QString& getUserName() { return(User); };
  inline void setUserName(QString s) { cerr << "USER: " << s << endl; User = s; };
  inline const char *getUserNameQs() { return(User.ascii()); };
  inline const char *getRoomName() { return(Room); };
  inline QString getRoomNameQs() { return(QString(Room)); };
  inline int getHndl() { return(g_cxHndl); };
  inline void setHndl( int i ) { g_cxHndl = i; };
  inline DCOPClient *dcopClient() { return(dcop); };

  enum View {
   InboxView,
   CalendarView,
   ContactsView,
   TasksView,
   JournalView,
   NotesView,
   TrashView,
   OtherView
  };

public slots:
  void 		slotAbout();
  void 		initializeClient();
  void		gotoRoom(QString);
  void		slotToggleKoolBar();
  void		slotToggleTitleBar();
  void		slotKoolBar(int,int);
  void		slotRefreshVisiblePane();

private slots:
  void		processTbEvent(int);
  void		itemSelected(QListViewItem *);

  void		slotUpdateToolbar();
  void 		slotDisconnect();
  void 		slotExit();
  void 		slotNext();
  void 		slotChat();

  void		slotNewMessage();
  void		slotNewNote();
  void		slotNewTask();

  void		slotKoInbox();
  void		slotKoCalendar();
  void		slotKoContacts();
  void		slotKoTasks();
  void		slotKoJournal();
  void		slotKoNotes();
  void		slotKoDeleted();
  void		slotKoByName( const QString& );
  void		slotKoExternal( int );
  void		slotKoExternal( int, int );

  void		slotUpdateLstFloors(const QString &which);
  void		slotToggleRooms();
  void		slotToggleMessagePane();
  void		slotToggleMessageHdr();
  void		slotToggleStatusBar();
  void		slotToggleToolbar();
  void		slotStartDaemon();

private:
  void		readProperties(KConfig *sessionConfig);
  void		saveProperties(KConfig *sessionConfig);
  void		LoadComponent( CXOBJDEF, QWidget *, const char *, CxObjGenericView * );
  void		LoadComponent( const QString&, const QString&, const QString&, QWidget * );
  void		UnloadComponent( const QString & );

  /**
   ** User Interface components.
   **/
  View			currentView;
  QSplitter		*Shoriz;
  QWidgetStack		*i_widgetStack;
  CxObjTitleBar		*i_titleBar;
  CxObjFolderList	*i_folderList;
#ifdef			HAVE_KOOLBAR
  KoKoolBar		*i_koolBar;
  QHBox			*i_kbContainer;
#endif

  /**
   ** This experimental snippet of code will be
   ** the foundation of the NEW way of doing
   ** program expansion.  Basically, each View/Pane
   ** will inherit the general properties of a 
   ** GenericView, allowing us to specify an arbitrary
   ** number of Views.  I hope. :)
   **/
  QDict<CxObjGenericView>	i_genericViews;
  QStringList			i_goPos;
#ifdef	HAVE_KOOLBAR
  QStringList			i_kbPos;
#endif

  /**
   ** Menu components.
   **/
  KMenuBar 	*i_menu;
  QPopupMenu 	*m_file,
		*m_view,
		*m_go,
		*m_compose,
		*m_tb_new;
  QPopupMenu	*m_popUp_noItem,
		*m_popUp_item; 
  KHelpMenu	*m_help;
  int		m_id_view_statusbar,
		m_id_view_folderlist,
		m_id_view_mesgpane,
		m_id_view_mesghdr,
		m_id_view_infusionbar,
		m_id_view_titlebar;

  /**
   ** Local variables.
   **/
  int 		Connected;
  CXLIST	_FloorIndex,
  		_RoomIndex;
  char		Room[255];
  QString	User;

  int		tb_NewButtonId;
  int		g_cxHndl;

  DCOPClient	*dcop;

};

void            Serialize(const QString &, QString *);
#endif
