/**
 ** infusion - The Desktop Communications Revolution
 ** Copyright (c) 2000, SCCG/Flaming Sword Productions
 ** All Rights Reserved
 **
 ** Module: modules/ui/kde2/wndmesg.o
 ** Date: 2000-10-15
 ** Last Revision: 2000-10-15
 ** Description: Message composer.
 ** CVS: $Id: wndmesg.cpp,v 1.11 2001/08/28 02:54:07 brian Exp $
 **/
#include        <iostream>
#include        <dcopclient.h>
#include        <kapp.h>
#include        <qapplication.h>
#include        <qpushbutton.h>
#include        <qlayout.h>
#include        <qframe.h>
#include        <kglobal.h>
#include        <kiconloader.h>
#include        <ktoolbar.h>
#include        <kaccel.h>
#include        <klineedit.h>
#include        <kdebug.h>
#include        <kdialog.h>
#include        <klocale.h>
#include        <ktabctl.h>
#include        <kmessagebox.h>
#include        <qstring.h>
#include        <CxClient.h>
#include	"wndmain.h"
#include	"wndaddress.h"
#include	"wndmesg.h"
#include	"autoconf.h"

#define		ID_SEND		0x1

extern CxWndMain *wndMain;

/**
 ** CxWndMesg::CxWndMesg(): Constructor
 **/
		CxWndMesg::CxWndMesg(
				const QString& from, 
				const QString& to, 
				const QString& subj, 
				const QString& room, 
				const QString& body, 
				const QDateTime& datetime,
				bool is_composer_win)
		:KMainWindow(0, 0) {

	/**
	 ** Copy the arguments into our global data bank.
	 **/
	g_ComposerWin = (is_composer_win != 0);
	g_DateTime = datetime;
	g_Room = room;
	g_From = from;
	g_To = to;
	g_Cc = QString::null;
	g_Subject = subj;
	g_MessageBody = body;

	prepareUserInterface();

	ive_been_changed = 0;

	resize(
		QApplication::desktop()->width() * 0.45, 
		QApplication::desktop()->height() * 0.45
	);
	move(
		(QApplication::desktop()->width() - width())/2, 
		(QApplication::desktop()->height() - height())/2
	);
}

/**
 ** CxWndMesg::CxWndMesg(): Alternate constructor.
 **/
		CxWndMesg::CxWndMesg( const QString &room, int msg_id, bool is_composer_win ) {
QCString	replyType;
QByteArray	replyData;
 
	ive_been_changed = 0;
	g_ComposerWin = is_composer_win;

	/**
	 ** Encapsulate the note in a DCOP message, which will
	 ** be gingerly passed to the CxMessegner::deliverMessage()
	 ** call.
	 **/
	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	arg << QCString( room );
	arg << msg_id;

	/**
	 ** Load the requested message from the Infusion Daemon.  If the
	 ** message could not be open, abort bootstrap and display an error
	 ** message.  (Then, close)
	 **/
	if(kapp->dcopClient()->call("infusiond", "CxMessenger", "fetchMessage(QCString,unsigned int)", data,
					replyType, replyData)) {
 
		g_UnableToOpen = 0;

		QDataStream reply(replyData, IO_ReadOnly);
		if (replyType == "QCStringList" ) {
			QCStringList mesgInfo;
			reply >> mesgInfo;

			QCStringList::Iterator x;
			int i = 0;
			for(x = mesgInfo.begin(); x != mesgInfo.end(); ++x) {
				switch(i) {
					case(0): g_Room = (*x); break;
					case(1): /*g_DateTime = QDateTime(*x); ?? */ break;
					case(2): g_From = (*x); break;
					case(3): g_To = (*x); break;
					case(4): g_Cc = (*x); break;
					case(5): g_Subject = (*x); break;
					case(6): g_MessageBody = (*x); break;
					default: kdDebug() << "CxWndMesg::CxWndMesg(): excessive data!  my head hurts!" << endl; break;
				}
				++i;
			}

			if( (i == 7) ) {
				prepareUserInterface();

				resize(
					QApplication::desktop()->width() * 0.45, 
					QApplication::desktop()->height() * 0.45
				);
				move(
					(QApplication::desktop()->width() - width())/2, 
					(QApplication::desktop()->height() - height())/2
				);

			} else {
				g_UnableToOpen = EBADDATA;
				kdDebug() << "CxWndMesg::CxWndMesg(): DCOP communications failure (EBADDATA)" << endl;
			}

		/**
		 ** This is nothing that we can work with.
		 **/
		} else {
			g_UnableToOpen = EINVALIDTYPE;
			kdDebug() << "CxWndMesg::CxWndMesg(): DCOP communications failure (EINVALIDTYPE)" << endl;
		}

	/**
	 ** This is nothing that we can work with.
	 **/
	} else {
		g_UnableToOpen = ENODAEMON;
		kdDebug() << "CxWndMesg::CxWndMesg(): DCOP communications failure (FAILURE TO COMMUNICATE)" << endl;
	}
}

/**
 ** CxWndMesg::prepareUserInterface(): Build the window's user interface.
 **/
void		CxWndMesg::prepareUserInterface() {

	setCaption(g_Subject.isNull()?i18n("No Subject"):g_Subject);

        i_keys = new KAccel(this);
        i_keys->connectItem(KStdAccel::Close, this, SLOT(close()));
        i_keys->connectItem(KStdAccel::Quit, this, SLOT(close()));
 
        prepareMenuBar();

        i_page = new QVBox(this);
        i_page->setMargin(KDialog::marginHint());
        i_page->setSpacing(KDialog::spacingHint());

	if(g_ComposerWin) prepareComposerPane();
	else prepareReaderPane();
 
	i_txtBody = new QMultiLineEdit(i_page, "wndMesgBody");
	i_txtBody->setWordWrap(QMultiLineEdit::WidgetWidth);
	i_txtBody->setText(g_MessageBody);
	connect(i_txtBody, SIGNAL(textChanged()), this, SLOT(slotChanged()));

        setCentralWidget(i_page);

        i_keys->readSettings();
}

/**
 ** CxWndMesg::prepareReaderPane(): Prepare the message-pane for a Reader
 ** window.
 **/
void		CxWndMesg::prepareReaderPane() {

	toolBar(0)->insertButton(QString("mail_reply"),ID_SEND, true, i18n("Reply"), -1, KGlobal::instance());
	toolBar(0)->insertLineSeparator();
	toolBar(0)->insertButton(QString("mail_replyall"),ID_SEND, true, i18n("Reply to All"), -1, KGlobal::instance());
	toolBar(0)->setIconText( KToolBar::IconTextRight );
//tmp		connect(toolBar(0), SIGNAL(clicked(int)), this, SLOT(processTbEvent(int)));

	KTabCtl *tabs = new KTabCtl( i_page );
	QWidget *tb1 = new QWidget( tabs );
	tabs->addTab( tb1, i18n("Message") );
	QGridLayout *grid = new QGridLayout(tb1, 4, 2, KDialog::marginHint() ,KDialog::spacingHint() );
	grid->setColStretch(0, 15);
	grid->setColStretch(1, 85);

	QLabel *lblDate = new QLabel( tb1 );
	lblDate->setText(i18n("Date:"));
	lblDate->setFont(QFont("helvetica", 14, QFont::Bold));
	lblDate->setAlignment( AlignRight );
	lblDate->show();
	grid->addWidget(lblDate, 0, 0, 0);

	QLabel *txtDate = new QLabel( tb1 );
	txtDate->setText( g_DateTime.toString() );
	txtDate->setFont( QFont("helvetica", 14) );
	txtDate->show();
	grid->addWidget(txtDate, 0, 1, 0);

	QLabel *lblSender = new QLabel( tb1 );
	lblSender->setText(i18n("From:"));
	lblSender->setFont(QFont( "helvetica", 12, QFont::Bold));
	lblSender->setAlignment( AlignRight );
	lblSender->show();
	grid->addWidget(lblSender, 1, 0, 0);

	QLabel *txtSender = new QLabel( tb1 );
	txtSender->setText(g_From);
	txtSender->setFont(QFont( "helvetica", 12 ));
	txtSender->show();
	grid->addWidget(txtSender, 1, 1, 0);

	QLabel *lblRcpt = new QLabel( tb1 );
	lblRcpt->setText(i18n("To:"));
	lblRcpt->setFont(QFont( "helvetica", 12, QFont::Bold));
	lblRcpt->setAlignment( AlignRight );
	grid->addWidget(lblRcpt, 2, 0, 0);

	QLabel *txtRcpt = new QLabel( tb1 );
	txtRcpt->setText(g_To);
	txtRcpt->setFont(QFont( "helvetica", 12 ));
	grid->addWidget(txtRcpt, 2, 1, 0);

	QLabel *lblCc = new QLabel( tb1 );
	lblCc->setFont(QFont( "helvetica", 12, QFont::Bold));
	lblCc->setText(i18n("Cc:"));
	lblCc->setAlignment( AlignRight );
	lblCc->hide();

	QLabel *txtCc = new QLabel( tb1 );
	txtCc->setText(QString::null);
	txtCc->setFont(QFont( "helvetica", 12 ));
	txtCc->hide();

	QLabel *lblSubj = new QLabel( tb1 );
	lblSubj->setText(i18n("Subject:"));
	lblSubj->setFont(QFont( "helvetica", 12, QFont::Bold ));
	lblSubj->setAlignment( AlignRight );
	grid->addWidget(lblSubj, 3, 0, 0);

	QLabel *txtSubj = new QLabel( tb1 );
	txtSubj->setText(g_Subject);
	txtSubj->setFont(QFont( "helvetica", 12 ));
	grid->addWidget(txtSubj, 3, 1, 0);
}

/**
 ** CxWndMesg::prepareComposerPane(): Prepare the message-pane for a Composer
 ** window.
 **/
void		CxWndMesg::prepareComposerPane() {
int		i;

	i_keys->insertItem(i18n("SendMessage"), "SendMessage", CTRL+Key_Return);
	i_keys->insertItem(i18n("RunQuickly"), "RunQuickly", Key_Escape);
	i_keys->connectItem("SendMessage", this, SLOT(onSend()));
	i_keys->connectItem("RunQuickly", this, SLOT(close()));

	toolBar(0)->insertButton(QString("mail_send"),ID_SEND, true, i18n("Send Message"), -1, KGlobal::instance());
	toolBar(0)->insertLineSeparator();
	i = toolBar(0)->insertButton(QString("filesave"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);
	i = toolBar(0)->insertButton(QString("fileprint"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);
	toolBar(0)->insertLineSeparator();
	i = toolBar(0)->insertButton(QString("editcut"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);
	i = toolBar(0)->insertButton(QString("editcopy"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);
	i = toolBar(0)->insertButton(QString("editpaste"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);
	i = toolBar(0)->insertButton(QString("attach"),ID_SEND, true, QString::null, -1, KGlobal::instance());
	toolBar(0)->setItemEnabled(i, false);

	connect(toolBar(0), SIGNAL(clicked(int)), this, SLOT(processTbEvent(int)));
	toolBar(0)->setIconText( KToolBar::IconTextRight );
	toolBar(0)->setFullSize(false);

	KTabCtl *tabs = new KTabCtl(i_page);
	QWidget *tb1 = new QWidget(tabs);
	tabs->addTab(tb1, i18n("Message"));

	QGridLayout *grid = new QGridLayout(tb1,2,2,KDialog::marginHint() ,KDialog::spacingHint(),"wndMesgGrid");

	if( g_Room == "Mail" ) {
		g_Room = QString("_MAIL_");
	}

	if(g_Room == "_MAIL_") {
		QPushButton *lblRcpt = new QPushButton(i18n("To..."), tb1,"wndMesgLblRcpt");
		grid->addWidget(lblRcpt,1,1,0);
		connect(lblRcpt, SIGNAL(clicked()), this, SLOT(openAddresser()));

		i_lRcpt = new KLineEdit(g_To, tb1, "wndMesgRcpt");
		grid->addWidget(i_lRcpt,1,2,0);

		QPushButton *lblCc = new QPushButton(i18n("Cc..."), tb1, "wndMesg.lblCc");
		lblCc->hide();
		//grid->addWidget(lblCc,1,3,0);
		connect(lblCc, SIGNAL(clicked()), this, SLOT(openAddresser()));

		i_lCc = new KLineEdit(g_Cc, tb1, "wndMesg.txtCc");
		i_lCc->hide();
		//grid->addWidget(i_lCc,1,3,0);


	} else {
		i_lRcpt = NULL;
		QLabel *lblRoom = new QLabel(i18n("Room:"), tb1, "wndMesgLblRoom");
		grid->addWidget(lblRoom,1,1,0);

		QLabel *lneRoom = new QLabel(g_Room, tb1, "wndMesgLneRoom");
		grid->addWidget(lneRoom,1,2,0);
	}

	QLabel *lblSubj = new QLabel(i18n("Subject:"), tb1, "wndMesgLblSubj");
	grid->addWidget(lblSubj,2,1,0);

	i_lSubj = new KLineEdit(g_Subject, tb1, "wndMesgSubj");
	connect(i_lSubj, SIGNAL(textChanged(const QString &)), this, SLOT(slotUpdateCaption(const QString &)));
	grid->addWidget(i_lSubj,2,2,0);

	if(i_lRcpt) i_lRcpt->setFocus();
	else i_lSubj->setFocus();
}

/**
 ** CxWndMesg::prepareMenuBar(): Prepare the window's menubar.
 **/
void		CxWndMesg::prepareMenuBar() {
int		i;

	/**
	 ** Prepare the "&File" menu
	 **/
	m_file = new QPopupMenu();
	i = m_file->insertItem("&Send Message", this, SLOT(onSend()));
	i_keys->changeMenuAccel(m_file, i, "SendMessage");
	m_file->insertSeparator();
	m_file->insertItem("Ne&w...", NULL);
	m_file->insertSeparator();
	m_file->insertItem("&Save", NULL);
	m_file->insertItem("Save &As...", NULL);
	m_file->insertSeparator();
	m_file->insertItem("&Delete", NULL);
	m_file->insertItem("&Move to Folder...", NULL);
	m_file->insertItem("Cop&y to Folder...", NULL);
	m_file->insertSeparator();
	m_file->insertItem("&Print", NULL);
	m_file->insertSeparator();
	m_file->insertItem("&Properties", NULL);
	m_file->insertSeparator();
	m_file->insertItem("&Close", this, SLOT(slotClose()));  

	/**
	 ** Prepare the "&Edit" menu
	 **/

	/**
	 ** Prepare the "&Compose" menu
	 **/
	m_compose = new QPopupMenu();
	m_compose->insertItem(SmallIcon(i18n("filenew")), i18n("&New Mail Message"), this, SLOT(slotNewEmail()));
	m_compose->insertItem( i18n("New Po&st in This Folder"), this, SLOT(slotNewPost()));
	m_compose->insertItem( i18n("C&hoose Template..."), NULL);
	m_compose->insertItem( i18n("Ch&oose Form..."), NULL);
	m_compose->insertSeparator();
	m_compose->insertItem(SmallIcon(QString("mail_reply")), i18n("&Reply"), NULL);
	m_compose->insertItem(SmallIcon(QString("mail_replyall")), i18n("Reply to A&ll"), NULL);
	m_compose->insertItem(SmallIcon(QString("mail_forward")), i18n("For&ward"), NULL);

	/**
	 ** Prepare the "&Go" menu
	 **/
	m_go = new QPopupMenu();
        m_go->insertItem(SmallIcon(QString("kword")), i18n("Inbox"), NULL);
        m_go->insertItem(SmallIcon(QString("karm")), i18n("Calendar"), NULL);
        m_go->insertItem(SmallIcon(QString("ktalkd")), i18n("Contacts"), NULL);
        m_go->insertItem(SmallIcon(QString("klipper")), i18n("Tasks"), NULL);
        m_go->insertItem(SmallIcon(QString("korganizer")), i18n("Journal"), NULL);
        m_go->insertItem(SmallIcon(QString("knotes")), i18n("Notes"), NULL);
        m_go->insertItem(SmallIcon(QString("trashcan_empty")), i18n("Deleted Items"), NULL);

	/**
	 ** Prepare the "&Help" menu
	 **/
	m_help = helpMenu( QString::null );

	/**
	 ** Attach all menus to the parent window's menubar.
	 **/
	menuBar()->insertItem("&File", m_file);
	menuBar()->insertItem("&Go", m_go);
	menuBar()->insertItem("Co&mpose", m_compose);
	menuBar()->insertSeparator();
	menuBar()->insertItem("&Help", m_help);
}

/**
 ** CxWndMesg::slotChanged(): What if somebody edits us??
 **/
void		CxWndMesg::slotChanged() {

	ive_been_changed = 1; // ive_been_assaulted = 0;
}

/**
 ** CxWndMesg::onSend(): Send message
 **/
void		CxWndMesg::onSend() {
QCString	replyType;
QByteArray	replyData;

	if(!g_ComposerWin) return;

	if(!*(i_txtBody->text().ascii())) {
		int i = KMessageBox::questionYesNo(this,
                        i18n("Are you sure you wish to send a blank message?"),
                        i18n("Send blank message?"),
                        i18n("Yes"),
                        i18n("No"),
                        false
                );
		if(i!=3) return;
	}

	/**
	 ** NEW: Instead of directly sending this, build a QCStringList and
	 ** pass it to infusiond for delivery.  The infusiond process will
	 ** handle the posting/delivery of the message, and report any failures
	 ** back to us.
	 **
	 ** MESGINFO FIELDS:
	 **  {Room, From, To, Cc, Subject, Message}
	 ** deliverMessage() RETURNS:
	 **  0:= Success
	 **  (value):= Failure, code (value)
	 **/
	QCStringList mesgInfo;
	QByteArray data;
        QDataStream arg(data, IO_WriteOnly);
	mesgInfo.append( QCString( g_Room ));
	mesgInfo.append( QCString(g_DateTime.toString()) );
	mesgInfo.append( QCString( wndMain->getUserName() ));
	if(i_lRcpt) mesgInfo.append( QCString( i_lRcpt->text() ));
	else mesgInfo.append( QCString("") );
	mesgInfo.append( QCString("") ); // QCString( i_lCc->text() ) );
	mesgInfo.append( QCString( i_lSubj->text() ));
	mesgInfo.append( QCString( i_txtBody->text() ));
        arg << mesgInfo;

	if(!kapp->dcopClient()->call("infusiond", "CxMessenger", "deliverMessage(QCStringList)", data, 
			replyType, replyData)) {
                cerr << "Infusion CxWndMesg::onSend(): (deliverMessage()) DCOP communications may be broken!" << endl;

        } else {
		QDataStream reply(replyData, IO_ReadOnly);
		if (replyType == "int" ) {
			int result;
			reply >> result;
			if( result ) {
				KMessageBox::sorry(this,
	                        i18n("I was unable to send your message.  Please check the recipient's name and try again.\n\nError Code %1").arg(result),
	                        i18n("Unable to Send"), false );
			} else {
				close();
			}
		} else {
                	cerr << "Infusion CxWndMesg::onSend(): (deliverMessage()) DCOP communications may be broken!" << endl;
		}
	}
}

/**
 ** CxWndMesg::slotClose(): Exit
 **/
void		CxWndMesg::slotClose() {
int		i;

	if(ive_been_changed) {
		i = KMessageBox::questionYesNo(this,
				(g_ComposerWin?i18n("Are you sure you wish to cancel the\nchanges you made to this message?"):i18n("Are you sure you wish to abort\nthis message?")),
				i18n("Cancel changes?"),
				i18n("&Yes"),
				i18n("&No"),
				false
		);

		if(i != 3) {
			return;
		}

	}

	close();
}

/**
 ** CxWndMesg::processTbEvent(): What happens when button x is hit?
 **/
void		CxWndMesg::processTbEvent(int x) {

	switch(x) {
		case(ID_SEND):
			emit onSend();
			break;

		default:
			break;
	}
}

/**
 ** CxWndMesg::slotUpdateCaption(): Update the titlebar.
 **/
void		CxWndMesg::slotUpdateCaption(const QString &txt) {

	setCaption( (txt.isEmpty())?i18n("No Subject"):txt );
}

/**
 ** CxWndMesg::openAddresser(): Open an addresser window.
 **/
void		CxWndMesg::openAddresser() {

	CxWndAddress *adwin = new CxWndAddress(this, QString::null);
	connect(adwin, SIGNAL(setRcptAddr(const QString &)), this, 
			SLOT(setRcptAddr(const QString &)));
	adwin->show();
}

/**
 ** CxWndMesg::setRcptAddr(): Set the recipient's address (called by CxWndAddress)
 **/
void		CxWndMesg::setRcptAddr(const QString & s) {

	if(i_lRcpt) i_lRcpt->setText(s);
}

/**
 ** CxWndMesg::slotNewEmail(): Create a new window for an e-mail composition.
 **/
void		CxWndMesg::slotNewEmail() {

	CxWndMesg *emailWin = new CxWndMesg( QString::null, QString::null, QString::null, 
				QString("_MAIL_"), QString::null, QDateTime::currentDateTime(), true );
	emailWin->show();
}

/**
 ** CxWndMesg::slotNewPost(): Create a new window for a public message composition.
 **/
void		CxWndMesg::slotNewPost() {

	CxWndMesg *postWin = new CxWndMesg( QString::null, QString::null, QString::null, 
				QString(wndMain->getRoomName()), QString::null, QDateTime::currentDateTime(), true );
	postWin->show();
}
